
var gAlbumArtManagerPane = {
  openPrefsPane: function() {
    var windowMediator;
    var window;

    /* Get the main Songbird window. */
    windowMediator = Components.classes
                        ["@mozilla.org/appshell/window-mediator;1"].
                        getService(Components.interfaces.nsIWindowMediator);
    window = windowMediator.getMostRecentWindow("Songbird:Main");

    /* Open the preferences pane. */
    window.SBOpenPreferences("library_albumartmanager_pref_pane");    
  },
  
  /**
   *
   */
  checkDefaultCoverFile: function() {
    var coverFileFormat = document.getElementById("coverFileFormat");
    if (coverFileFormat.value == "") {
      coverFileFormat.value = "%album%";
    }
    return true;
  },
  
  /**
   * Enables/disables the folder field and Browse button based on whether a
   * default download directory is being used.
   */
  readUseDownloadDir: function ()
  {
    var downloadFolder = document.getElementById("aamDownloadFolder");
    var chooseFolder = document.getElementById("aamChooseFolder");
    var coverFileFormat = document.getElementById("aamCoverFileFormat");
    var preference = document.getElementById("songbird.albumartmanager.cover.usealbumlocation");
    downloadFolder.disabled = preference.value;
    chooseFolder.disabled = preference.value;
    coverFileFormat.disabled = preference.value;
    
    // don't override the preference's value in UI
    return undefined;
  },

  /**
   * Displays a file picker in which the user can choose the location where
   * downloads are automatically saved, updating preferences and UI in
   * response to the choice, if one is made.
   */
  chooseFolder: function ()
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    const nsILocalFile = Components.interfaces.nsILocalFile;

    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    var bundlePreferences = document.getElementById("albumartmanagerbundle");
    var title = bundlePreferences.getString("albumartmanager.prefs.chooseDownloadFolderTitle");
    fp.init(window, title, nsIFilePicker.modeGetFolder);
    fp.appendFilters(nsIFilePicker.filterAll);

    var folderListPref = document.getElementById("songbird.albumartmanager.cover.folderList");
    var currentDirPref = this._indexToFolder(folderListPref.value); // file
    var defDownloads = this._indexToFolder(1); // file

    // First try to open what's currently configured
    if (currentDirPref && currentDirPref.exists()) {
      fp.displayDirectory = currentDirPref;
    } // Try the system's home dir
    else if (defDownloads && defDownloads.exists()) {
      fp.displayDirectory = defDownloads;
    } // Fall back to Desktop
    else {
      fp.displayDirectory = this._indexToFolder(0);
    }

    if (fp.show() == nsIFilePicker.returnOK) {
      var file = fp.file.QueryInterface(nsILocalFile);
      var currentDirPref = document.getElementById("songbird.albumartmanager.cover.folder");
      currentDirPref.value = file;
      var folderListPref = document.getElementById("songbird.albumartmanager.cover.folderList");
      folderListPref.value = this._folderToIndex(file);
      // Note, the real prefs will not be updated yet, so dnld manager's
      // userDownloadsDirectory may not return the right folder after
      // this code executes. displayDownloadDirPref will be called on
      // the assignment above to update the UI.
    }
  },

  /**
   * Initializes the download folder display settings based on the user's
   * preferences.
   */
  displayDownloadDirPref: function ()
  {
    var folderListPref = document.getElementById("songbird.albumartmanager.cover.folderList");
    var bundlePreferences = document.getElementById("albumartmanagerbundle");
    var downloadFolder = document.getElementById("aamDownloadFolder");
    var currentDirPref = document.getElementById("songbird.albumartmanager.cover.folder");

    // The user's download folder is based on the preferences listed above.
    // However, if the system does not support a download folder, the
    // actual path returned will be the system's desktop or home folder.
    // If this is the case, skip off displaying the Download label and
    // display Desktop, even though folderList might be 1.
    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
                                .getService(Components.interfaces.nsIProperties);
    var desk = fileLocator.get("Desk", Components.interfaces.nsILocalFile);
    var home = fileLocator.get("Home", Components.interfaces.nsILocalFile);
    var dnldMgr = Components.classes["@mozilla.org/download-manager;1"]
                            .getService(Components.interfaces.nsIDownloadManager);
    var supportDownloadLabel = !dnldMgr.defaultDownloadsDirectory.equals(desk);

    // Used in defining the correct path to the folder icon.
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
    var fph = ios.getProtocolHandler("file")
                 .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var iconUrlSpec;

    // Display a 'pretty' label or the path in the UI.
    if (folderListPref.value == 2) {
      // Custom path selected and is configured
      downloadFolder.label = this._getDisplayNameOfFile(currentDirPref.value);
      iconUrlSpec = fph.getURLSpecFromFile(currentDirPref.value);
    } else if (folderListPref.value == 1 && supportDownloadLabel) {
      downloadFolder.label = bundlePreferences.getString("albumartmanager.prefs.homeFolderName");
      iconUrlSpec = fph.getURLSpecFromFile(home);
    } else {
      // 'Desktop'
      downloadFolder.label = bundlePreferences.getString("albumartmanager.prefs.desktopFolderName");
      iconUrlSpec = fph.getURLSpecFromFile(desk);
    }
    downloadFolder.image = "moz-icon://" + iconUrlSpec + "?size=16";

    // don't override the preference's value in UI
    return undefined;
  },

  _getDefaultMusicFolder: function()
  {
    const dirService = Cc["@mozilla.org/file/directory_service;1"].
                       getService(Ci.nsIDirectoryServiceProvider);
    const platform = Cc["@mozilla.org/xre/app-info;1"].
                     getService(Ci.nsIXULRuntime).
                     OS;

    var musicDir;
    switch (platform) {
      case "WINNT":
        musicDir = dirService.getFile("Pers", {});
        musicDir.append("My Music");
        break;

      case "Darwin":
        musicDir = dirService.getFile("Music", {});
        break;

      case "Linux":
        musicDir = dirService.getFile("Home", {});
        musicDir.append("Music");

        if (!folderIsValid(musicDir)) {
          musicDir = musicDir.parent;
          musicDir.append("music");
        }
        break;

      default:
        // Fall through and use the Desktop below.
        break;
    }

    // Make sure that the directory exists and is writable.
    if (!folderIsValid(musicDir)) {
      // Great, default to the Desktop... This should work on all OS's.
      musicDir = dirService.getFile("Desk", {});

      // We should never get something bad here, but just in case...
      if (!folderIsValid(musicDir)) {
        Cu.reportError("Desktop directory is not a directory!");
        throw Cr.NS_ERROR_FILE_NOT_DIRECTORY;
      }
    }

    return musicDir;
  },

  /**
   * Returns the textual path of a folder in readable form.
   */
  _getDisplayNameOfFile: function (aFolder)
  {
    // TODO: would like to add support for 'Downloads on Macintosh HD'
    //       for OS X users.
    return aFolder ? aFolder.path : "";
  },

  /**
   * Returns the Downloads folder.  If aFolder is "Desktop", then the Downloads
   * folder returned is the desktop folder; otherwise, it is the home folder.
   *
   * @throws if aFolder is not "Desktop" or "Home"
   */
  _getDownloadsFolder: function (aFolder)
  {
    switch(aFolder)
    {
      case "Desktop":
        var fileLoc = Components.classes["@mozilla.org/file/directory_service;1"]
                                    .getService(Components.interfaces.nsIProperties);
        return fileLoc.get("Desk", Components.interfaces.nsILocalFile);
      break;
      case "Home":
        var fileLoc = Components.classes["@mozilla.org/file/directory_service;1"]
                                    .getService(Components.interfaces.nsIProperties);
        return fileLoc.get("Home", Components.interfaces.nsILocalFile);
      break;
    }
    throw "ASSERTION FAILED: folder type should be 'Desktop' or 'Home'";
  },

  /**
   * Determines the type of the given folder.
   *
   * @param   aFolder
   *          the folder whose type is to be determined
   * @returns integer
   *          0 if aFolder is the Desktop or is unspecified,
   *          1 if aFolder is the Home folder,
   *          2 otherwise
   */
  _folderToIndex: function (aFolder)
  {
    if (!aFolder || aFolder.equals(this._getDownloadsFolder("Desktop")))
      return 0;
    else if (aFolder.equals(this._getDownloadsFolder("Home")))
      return 1;
    return 2;
  },

  /**
   * Converts an integer into the corresponding folder.
   *
   * @param   aIndex
   *          an integer
   * @returns the Desktop folder if aIndex == 0,
   *          the Home folder if aIndex == 1,
   *          the folder stored in songbird.albumartmanager.cover.folder
   */
  _indexToFolder: function (aIndex)
  {
    switch (aIndex) {
      case 0:
        return this._getDownloadsFolder("Desktop");
      case 1:
        return this._getDownloadsFolder("Home");
    }
    var currentDirPref = document.getElementById("songbird.albumartmanager.cover.folder");
    return currentDirPref.value;
  },

  /**
   * Returns the value for the songbird.albumartmanager.cover.folderList preference.
   */
  getFolderListPref: function ()
  {
    var folderListPref = document.getElementById("songbird.albumartmanager.cover.folderList");
    switch(folderListPref.value) {
      case 0: // Desktop
      case 1: // Home
        return folderListPref.value;
      break;
      case 2: // Custom
        var currentDirPref = document.getElementById("songbird.albumartmanager.cover.folder");
        if (currentDirPref.value) {
          // Resolve to a known location if possible. We are writing out
          // to prefs on this call, so now would be a good time to do it.
          return this._folderToIndex(currentDirPref.value);
        }
        return 0;
      break;
    }
  }
};
