/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/


/* This XPCOM service knows how to ask Amazon.com where to find art for 
 * music albums. 
 */

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;


Components.utils.import('resource://app/components/sbProperties.jsm');

function sbAmazonCoverFetcher() {
  this._ioService = Cc['@mozilla.org/network/io-service;1']
    .getService(Ci.nsIIOService);
  this._shutdown = false;
}
sbAmazonCoverFetcher.prototype = {
  classDescription: 'Songbird Amazon Album Cover Fetcher',
  classID: Components.ID('{958e914f-84d6-4501-aa0c-17c575de0dd1}'),
  contractID: '@songbirdnest.com/Songbird/cover/amazon-fetcher;1'
}

// nsISupports
sbAmazonCoverFetcher.prototype.QueryInterface =
function sbAmazonCoverFetcher_QueryInterface(aIID) {
  if(!aIID.equals(Ci.nsISupports) &&
     !aIID.equals(Ci.sbICoverFetcher)) {
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
  return this;
}

sbAmazonCoverFetcher.prototype._findCoverURL = 
function sbAmazonCoverFetcher_findCoverURL(aMediaItem, aCallback) {
  /* build the amazon query */
  var amazonKey = '0ACF688NVCRN8HN5FGG2';
  var amazonTag = 'albumartmanager-sb';
  var amazonTld = 'com';
  var amazonURL = 'http://webservices.amazon.' + amazonTld + 
    '/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=' + amazonKey + 
    '&AssociateTag=' + amazonTag + '&Operation=ItemSearch&SearchIndex=Music';

  var artist = aMediaItem.getProperty(SBProperties.artistName);
  if (artist && artist != '') {
    amazonURL += '&Artist=' + escape(artist);
  }

  var album = aMediaItem.getProperty(SBProperties.albumName);
  if (artist && artist != '') {
    amazonURL += '&Title=' + escape(album);
    amazonURL += '&ResponseGroup=Small,Images';
  } else {
    amazonURL += '&ResponseGroup=Small,Images,Tracks';
  }


  var xhr = Cc['@mozilla.org/xmlextras/xmlhttprequest;1']
    .createInstance(Ci.nsIXMLHttpRequest);
  xhr.open('get', amazonURL, true);
  xhr.onreadystatechange = function() {
    if (xhr.readyState == 4) {
      // for bug in mozilla that throw error when calling status
      try {
        xhr.status;
      } catch (e) {
        aCallback(null);
        xhr.cancel();
        return;
      }

      if (xhr.status != 200) {
        // a non-200 response is considered an error
        aCallback(null);
        return;
      }

      if (!Components.isSuccessCode(xhr.channel.status)) {
        // make sure we weren't aborted
        aCallback(null);
        return;
      }

      if (!xhr.responseXML) {
        // the response has to be XML to be useful to us
        aCallback(null);
        return;
      }

      // so at this point we have a 200 response with XML data

      // FIXME: move this to XPath

      // find all the items in the response
      var elements = xhr.responseXML.getElementsByTagName('Item');
      // what order do we look for the sizes?
      var sizes = ['Large', 'Medium', 'Small'];
      // for each element, in relevance order...
      for (var i=0; i<elements.length; i++) {
        var element = elements[i];
        // look for the size of image we want
        for (var j=0; j<sizes.length; j++) {
          var url = null;
          try {
            url = 
              // find the first <size>Image element
              element.getElementsByTagName(sizes[j]+'Image')[0]
              // find the first URL element
              .getElementsByTagName('URL')[0]
              // get it's contents as text
              .firstChild.nodeValue;
            // looks like we found it
          } catch (e) {
            // no problem, we'll try another way
          }
          if (url) {
            aCallback(url);
            return;
          }
        }
      }

      // we didn't find an appropriate image
      aCallback(null);
      return;
    }
  }
  xhr.send(null);
}
 
// sbICoverFetcher
sbAmazonCoverFetcher.prototype.fetchCoverForMediaItem = 
function sbAmazonCoverFetcher_fetchCoverForMediaItem(aMediaItem, aListener) {
  if (this._shutdown) {
    return;
  }

  var contentURI = this._ioService.newURI(
      aMediaItem.getProperty(SBProperties.contentURL), null, null);

  // if the URL isn't valid, fail
  if (!contentURI) {
    aListener.coverFetchFailed(aMediaItem, null);
    return;
  }

  // if the URL isn't local, fail
  var fileURL = contentURI.QueryInterface(Ci.nsIFileURL);
  if (!fileURL) {
    aListener.coverFetchFailed(aMediaItem, null);
    return;
  }

  // get the directory
  var path = fileURL.file.parent;

  // add 'cover.jpg' as the file name for cover images
  path.append('cover.jpg');

  // the result is scoped to the artist & album
  var scope = Cc['@songbirdnest.com/Songbird/Properties/MutablePropertyArray;1']
    .createInstance(Ci.sbIMutablePropertyArray);
  scope.appendProperty(SBProperties.albumName, 
      aMediaItem.getProperty(SBProperties.albumName));
  scope.appendProperty(SBProperties.artistName, 
      aMediaItem.getProperty(SBProperties.artistName));

  this._findCoverURL(aMediaItem, function(aURL) {
    /* FIXME: scope? */
    if (!aURL) {
      // epic fail
      aListener.coverFetchFailed(aMediaItem, scope);
    } else {
      aListener.coverFetchSucceeded(aMediaItem, scope, aURL);
    }
  });
}

sbAmazonCoverFetcher.prototype.shutdown =
function sbAmazonCoverFetcher_shutdown() {
  this._shutdown = true;
}


Components.utils.import('resource://gre/modules/XPCOMUtils.jsm');

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([sbAmazonCoverFetcher]);
}




