/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/


/* This XPCOM service knows how to find album art that's stored on your
 * local disk.
 */

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;


Components.utils.import("resource://app/components/sbProperties.jsm");


function sbFileCoverFetcher() {
  this._ioService = Cc['@mozilla.org/network/io-service;1']
    .getService(Ci.nsIIOService);
  this._albumArtService = Cc['@songbirdnest.com/songbird-album-art-service;1']
    .getService(Ci.sbIAlbumArtService);
  this._shutdown = false;
}
sbFileCoverFetcher.prototype = {
  classDescription: "Songbird Local File Album Cover Fetcher",
  classID: Components.ID("{ca2da771-c58a-44cf-8b58-139f553d62a1}"),
  contractID: "@songbirdnest.com/Songbird/cover/file-fetcher;1"
}

// nsISupports
sbFileCoverFetcher.prototype.QueryInterface =
function sbFileCoverFetcher_QueryInterface(aIID) {
  if(!aIID.equals(Ci.nsISupports) &&
     !aIID.equals(Ci.sbICoverFetcher)) {
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
  return this;
}

// sbICoverFetcher
sbFileCoverFetcher.prototype.fetchCoverForMediaItem = 
function sbFileCoverFetcher_fetchCoverForMediaItem(aMediaItem, aListener) {
  if (this._shutdown) {
    return;
  }

  // where would we store this media item's cover?
  var location = this._albumArtService.getCoverDownloadLocation(aMediaItem);

  var extensions = ['.jpg','.jpeg', '.png', '.gif', '.tif'];

  if (location) {
    for (var i=0; i<extensions.length; i++) {
      var file = Cc['@mozilla.org/file/local;1']
        .createInstance(Ci.nsILocalFile);
      try {
        var path = location+extensions[i];
        file.initWithPath(path);
      } catch(e) {
        Components.utils.reportError(e);
        aListener.coverFetchFailed(aMediaItem, null);
        return;
      }
      if (file.exists()) {
        aListener.coverFetchSucceeded(aMediaItem, null, 
            this._ioService.newFileURI(file).spec);
        return;
      }
    }
    aListener.coverFetchFailed(aMediaItem, null);
  } else {
    aListener.coverFetchFailed(aMediaItem, null);
  }
}

// sbICoverFetcher
sbFileCoverFetcher.prototype.shutdown =
function sbFileCoverFetcher_shutdown() {
  this._shutdown = true;
}

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([sbFileCoverFetcher]);
}



