// AMTB_Search will perform the GET request to search allmusic.com
function AMTB_Search()
{
    // This variable will hold the text of the search box
    var URL = "";

    // This variable will tell us whether our search box is empty or not
    var isEmpty = false;

    // Handle for Preferences & Search Terms Box
    var searchTermsBox = document.getElementById("AMTB-SearchTerms");
    var searchPrefBox = document.getElementById("AMTB-SearchPref");
    
    // Get Search Preference into URI safe text
    var searchPref = AMTB_TrimString(searchPrefBox.value);
    searchPref = AMTB_ConvertTermsToURI(searchPref);
    
    // Get the value in the search terms box, trimming whitespace as necessary
    // with AMTB_TrimString() 
    var searchTerms = AMTB_TrimString(searchTermsBox.value);
    var searchTermsPreTrim = searchTermsBox.value;
    
    if(searchTerms.length == 0) // Is the search terms box empty?
        isEmpty = true;         // If so, set the isEmpty flag to true
    else                        // If not, convert the terms to a URL-safe string
        searchTerms = AMTB_ConvertTermsToURI(searchTerms);

	// Build URL with searchPref and searchTerms variables else go to allmusic.com if empty
	if(isEmpty) { URL = "http://www.allmusic.com/"; }
	else        { URL = "http://www.allmusic.com/cg/amg.dll?P=amg&opt1=" + searchPref + "&sql=" + searchTerms; }
	    
    // Load the URL in the browser window using the AMTB_LoadURL function
    AMTB_LoadURL(URL);
    
    // Add to pulldown menu for history
    if (searchTerms != "") {AMTB_Populate(searchTermsPreTrim);}
}

////////////////////////////////////////////////////////////////////////////////
// The AMTB_TrimString() function will trim all leading and trailing whitespace
// from the incoming string, and convert all runs of more than one whitespace
// character into a single space. The altered string gets returned.
////////////////////////////////////////////////////////////////////////////////
function AMTB_TrimString(string)
{
    // If the incoming string is invalid, or nothing was passed in, return empty
    if (!string)
        return "";

    string = string.replace(/^\s+/, ''); // Remove leading whitespace
    string = string.replace(/\s+$/, ''); // Remove trailing whitespace

    // Replace all whitespace runs with a single space
    string = string.replace(/\s+/g, ' ');

    return string; // Return the altered value
}

////////////////////////////////////////////////////////////////////////////////
// The AMTB_ConvertTermsToURI() function converts an incoming string of search
// terms to a safe value for passing into a URL for the GET request.
////////////////////////////////////////////////////////////////////////////////
function AMTB_ConvertTermsToURI(terms)
{
    // Create an array to hold each search term
    var termArray = new Array();

    // Split up the search term string based on the space character
    termArray = terms.split(" ");

    // Create a variable to hold our resulting URI-safe value
    var result = "";

    // Loop through the search terms
    for(var i=0; i<termArray.length; i++)
    {
        // All search terms (after the first one) are to be separated with a HTML space
        if(i > 0)
            result += "%20";

        // Encode each search term, using the built-in Firefox function
        // encodeURIComponent().
        result += encodeURIComponent(termArray[i]);
    }

    return result; // Return the result
}

////////////////////////////////////////////////////////////////////////////////
// The AMTB_LoadURL() function loads the specified URL in the browser.
////////////////////////////////////////////////////////////////////////////////
function AMTB_LoadURL(url)
{
    // Set the browser window's location to the incoming URL
    window._content.document.location = url;

    // Make sure that we get the focus
    window.content.focus();
}

//Allow user to press enter to submit
function AMTB_KeyHandler(event)
{
    if(event.keyCode == event.DOM_VK_RETURN)
        AMTB_Search();
}

////////////////////////////////////////////////////////////////////////////////
// The AMTB_Populate() function places all search text within the toolbar's 
// drop-down input box for a complete history of searches for that session. 
////////////////////////////////////////////////////////////////////////////////
function AMTB_Populate(searchTermsPreTrim)
{		
	// Get the menupopup element that we will be working with
    var menu = document.getElementById("AMTB-SearchTermsMenu");

	// Create a new menu item to be added
	var tempItem = document.createElement("menuitem");
	
	// Set the new menu item's label
	tempItem.setAttribute("label", searchTermsPreTrim);
       
 	// Add the item to our menu
    menu.appendChild(tempItem);
}

function Context_Search(type)
{
	// Get selected text, type, & build URL
	var selected_text = window.content.getSelection();
	var searchPref = type;
	var URL = "http://www.allmusic.com/cg/amg.dll?P=amg&opt1=" + searchPref + "&sql=" + selected_text;
	
	if(selected_text == "") { URL = "http://www.allmusic.com/"; }
		
	// Load the URL in the browser window using the AMTB_LoadURL function
    AMTB_LoadURL(URL);
    
    // Add to pulldown menu as history
     if (selected_text != "") {AMTB_Populate(selected_text)};
}