const usernamePref	= "extensions.audioscrobbler.username";
const passwordPref	= "extensions.audioscrobbler.password";
const SB_NS = "http://songbirdnest.com/data/1.0#";
var Cc = Components.classes;
var Ci = Components.interfaces;
var initial = true;
var currentlyPlayingMediaItemGuid;
var trackStartedPlaying;
var mainLib;
var scrobblerTimer;

document.addEventListener("load", onload, true);

var trackChange = {
	observe:function(subject, topic, data) {
		// Don't care, just quit
		quitSongbird();
	}
}

// Rules
// Can call scrobbleNowPlaying() upon track playback initiation
// Can only call scrobbleTrack() upon track completion if the track:
// 		a) has been played for min(total_duration/2, 240s)
//		b) total_duration > 30s
var playingListener = {
	as: audioscrobbler,
	gpps: gPPS,

	observe: function(aSubject, aTopic, aData) {
		if (aTopic != "faceplate.playing" || !this.as.loggedIn())
			return;

		// Clear the timeout for the scrobbleTrack
		clearTimeout(scrobblerTimer)
		if (aData != "1") {
			return;
		}

		var currentGuid = this.gpps.currentGUID;
		var mediaItem = mainLib.getMediaItem(currentGuid);
		currentlyPlayingMediaItemGuid = currentGuid;
		var artist = mediaItem.getProperty(SB_NS + "artistName");
		var album = mediaItem.getProperty(SB_NS + "albumName");
		var title = mediaItem.getProperty(SB_NS + "trackName");
		var trackNum = mediaItem.getProperty(SB_NS + "trackNumber");
		var duration = mediaItem.getProperty(SB_NS + "duration");

		duration = parseInt(duration/1000000);
		this.as.scrobbleNowPlaying(artist, title, album, duration, trackNum);

		// Constraint (b)
		if (duration < 30)
			return;

		var min = 240;
		if (duration/2 < min)
			min = duration/2;
		
		var curTime = new Date();
		var ts = parseInt(curTime.getTime()/1000);
		trackStartedPlaying = ts;

		// Constraint (a)
		scrobblerTimer = setTimeout("scrobbleTrack()", min*1000);
	}
}

function scrobbleTrack() {
	var mediaItem = mainLib.getMediaItem(currentlyPlayingMediaItemGuid);
	var artist = mediaItem.getProperty(SB_NS + "artistName");
	var album = mediaItem.getProperty(SB_NS + "albumName");
	var title = mediaItem.getProperty(SB_NS + "trackName");
	var trackNum = mediaItem.getProperty(SB_NS + "trackNumber");
	var duration = mediaItem.getProperty(SB_NS + "duration");

	duration = parseInt(duration/1000000);
	audioscrobbler.scrobbleTrack(artist, title, album, duration, trackNum,
			trackStartedPlaying);
}

function onload(evt) {
	if ((evt.originalTarget.nodeName != "#document") || !initial)
		return;

	// Don't call this again
	initial = false;

	// Get our reference to the library
	var libraryManager = Cc["@songbirdnest.com/Songbird/library/Manager;1"].
				getService(Ci.sbILibraryManager);
	mainLib = libraryManager.mainLibrary;
	
	// Add a listener for track change
    var playingRemote = SB_NewDataRemote( "faceplate.playing", null );
    playingRemote.bindObserver(playingListener, true);
		
	// Grab our preferences
	var prefs = Cc["@mozilla.org/preferences-service;1"].
		getService(Ci.nsIPrefBranch2);
	var username = prefs.getCharPref(usernamePref);
	var password = prefs.getCharPref(passwordPref);

	try {
		audioscrobbler.Init(username, password);
	} catch (e) {
		alert("Failed to initialise Audioscrobbler: " + e);
		return;
	}
}
