var Cc = Components.classes;
var Ci = Components.interfaces;
var debugPref	= "extensions.audioscrobbler.debug";
const asClient = "tst";
const asVersion = "1.0";

// the URL we do the initial handshake with in Init()
const handshakeURL =
	"http://post.audioscrobbler.com/?hs=true&p=1.2&c=" + asClient + "&v=" +
	asVersion;

function AudioscrobblerException (message, hardFailure, retryTime)
{
  this.message=message;
  this.name="AudioscrobblerException";
  this.hardFailure = hardFailure;
  if (hardFailure)
	  this.retryTime = -1;
  else
	  this.retryTime = retryTime;
}

AudioscrobblerException.prototype.toString = function ()
{
  return this.name + ': "' + this.message + '"';
}

var audioscrobbler = {
	// the session ID we get back from Audioscrobbler
	// null if we're not logged in
	_sessionID : null,

	// the submission URL we get back from Audioscrobbler
	_submitURL : null,

	// the nowplaying URL we get back from Audioscrobbler
	_nowPlayingURL : null,

	// whether to log debug msgs or not
	_debug : false,
	_console: null,

	_username : null,
	_password : null
};

audioscrobbler.handleResponse = function(responseText) {
	var response = responseText.split("\n", 4);
	this.DebugLog("handshake response: " + response[0]);
	switch(response[0]) {
		case "OK" : 
			this._sessionID = response[1];
			this._nowPlayingURL = response[2];
			this._submitURL = response[3];
			this.DebugLog("handshake successful");
			this.DebugLog("handshake session ID: " + this._sessionID);
			this.DebugLog("handshake nowplaying url: " + this._nowPlayingURL);
			this.DebugLog("handshake submission url: " + this._submitURL);
			break;
		case "BANNED" :
			throw new AudioscrobblerException("Banned client", true);
			break;
		case "BADAUTH" :
			throw new AudioscrobblerException("Unauthorised", true);
			break;
		case "BADTIME" :
			throw new AudioscrobblerException("Bad time", true);
			break;
		default:
			throw new AudioscrobblerException("Failed", true);
			break;
	}
	return true;
}

audioscrobbler.DebugLog = function(msg) {
	if (this._debug)
		this._console.logStringMessage("Audioscrobbler: " + msg);
}

audioscrobbler.Init = function(username, password, async) {
	var curTime = new Date();
	this._username = username;
	this._password = password;

	var prefs = Cc["@mozilla.org/preferences-service;1"].
		getService(Ci.nsIPrefBranch2);
	if (prefs.getPrefType(debugPref) == prefs.PREF_BOOL)
		this._debug = prefs.getBoolPref(debugPref);
	else {
		this._debug = false;
		prefs.setBoolPref(debugPref, false);
	}
	this._console = Cc["@mozilla.org/consoleservice;1"].
		getService(Ci.nsIConsoleService);

	var ts = parseInt(curTime.getTime()/1000);
	var auth = MD5(MD5(password) + ts);
	var hsURL = handshakeURL + "&u=" + username + "&t=" + ts + "&a=" + auth;

	this.DebugLog("handshake URL: " + hsURL);
	this.DebugLog("handshake mode: " + (async == null ? "sync" : "sync"));

	if (async == null)
		async = "true";
	var req = new XMLHttpRequest();
	req.open("GET", hsURL, async);
	req.as = this;
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			this.as.handleResponse(this.responseText);
		}
	}
	req.send(null);
	if (!async)
		return this.handleResponse(req.responseText);
}

audioscrobbler.loggedIn = function() {
	return (this._sessionID != null);
}

audioscrobbler.scrobbleNowPlaying = function(artist, track, album, duration,
		trackNum) {
	// Can only scrobble if we're logged in
	if (!this.loggedIn())
		return false;

	var params = "s=" + this._sessionID + "&a=" + encodeURIComponent(artist) +
				"&t=" + encodeURIComponent(track) + "&b=" +
				encodeURIComponent(album) + "&l=" + duration + "&n=" +
				trackNum + "&m=";
	this.DebugLog("NowPlaying post url: " + this._nowPlayingURL);
	this.DebugLog("NowPlaying post data: " + params);
	var req = new XMLHttpRequest();
	req.as = this;
	req.open("POST", this._nowPlayingURL, true);
	req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
	req.setRequestHeader("Content-length", params.length);
	req.setRequestHeader("Connection", "close");
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			var response = this.responseText.split("\n", 1);
			this.as.DebugLog("NowPlaying response: " + response[0]);
			if (response[0] == "OK") {
				//alert("scrobbled now playing, result: " + this.responseText);
				// Do nothing
			} else {
				// XXX should try to auto rehandshake and renegotiate
				throw new AudioscrobblerException("Bad session", true);
			}
		}
	}
	req.send(params);
}

audioscrobbler.scrobbleTrack = function(artist, track, album, duration,
		trackNum, startTime) {
	var source = "P";
	var rating = "L";

	// Can only scrobble if we're logged in
	if (!this.loggedIn())
		return false;

	var params = "s=" + this._sessionID + "&a[0]=" +
			encodeURIComponent(artist) + "&t[0]=" + encodeURIComponent(track) +
			"&b[0]=" + encodeURIComponent(album) + "&l[0]=" + duration +
			"&n[0]=" + trackNum + "&i[0]=" + startTime + "&o[0]=" + source +
			"&r[0]=" + rating + "&m[0]=";

	this.DebugLog("Scrobble post url: " + this._submitURL);
	this.DebugLog("Scrobble post data: " + params);
	var req = new XMLHttpRequest();
	req.as = this;
	req.open("POST", this._submitURL, true);
	req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
	req.setRequestHeader("Content-length", params.length);
	req.setRequestHeader("Connection", "close");
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			var response = this.responseText.split("\n", 1);
			this.as.DebugLog("Scrobble response: " + response[0]);
			if (response[0] == "OK") {
				// Do nothing
				//alert("track scrobbled " + this.responseText);
			} else {
				// XXX should try to auto rehandshake and renegotiate
				switch (response[0]) {
					case "BADSESSION":
						throw new AudioscrobblerException("Bad session", true);
						break;
					default:
						throw new AudioscrobblerException("Failed", true);
				}
			}
		}
	}
	req.send(params);
}

// Utility function to compute the MD5sum of a string
function MD5(s) {
	var hash = Cc["@mozilla.org/security/hash;1"]
			   .createInstance(Ci.nsICryptoHash);
	hash.init(Ci.nsICryptoHash.MD5);

	var ss = Cc["@mozilla.org/io/string-input-stream;1"]
			 .createInstance(Ci.nsIStringInputStream);
	ss.setData(s, s.length);
	hash.updateFromStream(ss, s.length);
	return ToHex(hash.finish(false)).toLowerCase();
}
function ToHex(s) {
	var hexchars = '0123456789ABCDEF';
	var hexrep = new Array(s.length * 2);

	for (var i = 0; i < s.length; ++i) {
		hexrep[i * 2] = hexchars.charAt((s.charCodeAt(i) >> 4) & 0xf);
		hexrep[i * 2 + 1] = hexchars.charAt(s.charCodeAt(i) & 0xf);
	}
	return hexrep.join("");
}

