/*
//
// BEGIN SONGBIRD GPL
// 
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2007 POTI, Inc.
// http://songbirdnest.com
// 
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
// 
// Software distributed under the License is distributed 
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either 
// express or implied. See the GPL for the specific language 
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this 
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc., 
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
// 
// END SONGBIRD GPL
//
 */


// Make a namespace.
if (typeof SBDevTools == 'undefined') {
  var SBDevTools = {};
}

// Create the controller if it doesn't already exist
if (typeof SBDevTools.Controller == 'undefined') {

  /**
   * Menu functions
   */
  SBDevTools.Controller = {
    
    init: function() {
      SBDevTools.RestartHelper.processDeferredActions();
      
      // Set the show profile manager toggle 
      var profileService = Components.classes["@mozilla.org/toolkit/profile-service;1"]
                                     .getService(Components.interfaces.nsIToolkitProfileService);
      var profManItem = document.getElementById("sbdevtools-profilemanager-menuitem");
      profManItem.setAttribute("checked", !profileService.startWithLastProfile);
      
      // Set the disable xul cache menu item
      if (Application.prefs.has("nglayout.debug.disable_xul_cache")) {
        var xulCacheItem = document.getElementById("sbdevtools-xulcache-menuitem");
        var xulCacheEnabled = Application.prefs.get("nglayout.debug.disable_xul_cache").value == true; 
        xulCacheItem.setAttribute("checked", xulCacheEnabled);
      }
            
      this.setUpBugButton();
    },

    
    /**
     * Force the current window to reload
     */
    refreshUI: function() {
      var feathersManager = Components.classes['@songbirdnest.com/songbird/feathersmanager;1']
                                     .getService(Components.interfaces.sbIFeathersManager);
      var layoutURL = feathersManager.currentLayoutURL;
      var skinName = feathersManager.currentSkinName;
      
      // Hack:  In order for images to reload we need to force
      // the skin pref to notify observers. 
      // The real value will be restored by the feathers manager
      // As soon as it completes the switch.
      
      Application.prefs.setValue("general.skins.selectedSkin", "rubberducky/0.2");
      
      setTimeout(function() {
          feathersManager.switchFeathers(layoutURL, skinName);
        }, 100);
    },
    
    
    /**
     * Open the feathers wizard
     */
    launchFeathersWizard: function() {
      window.openDialog("chrome://sbdevtools/content/feathers-wizard.xul", 
        "feathers-wizard", "centerscreen");
    },
    
    
    /**
     * Open the extension wizard
     */
    launchExtensionWizard: function() {
      window.openDialog("chrome://sbdevtools/content/extension-wizard.xul", 
        "extension-wizard", "centerscreen");
    },
    

    /**
     * Open the js environment
     */
    launchJavascriptShell: function() {
      window.openDialog("chrome://extensiondev/content/jsenv.xul", "_blank", "all=no,width=500,height=400,scrollbars=yes,resizable=yes,dialog=no");
      //window.openDialog("chrome://extensiondev/content/shell.xul", "_blank", "all=no,width=500,height=400,scrollbars=yes,resizable=yes,dialog=no");
    },
    

    /**
     * Open the js debugger
     */
    launchJavascriptDebugger: function() {
      loadURI("x-jsd:debugger");
    },
 
    
    /** 
     * Show the XUL Periodic Table, an example page that 
     * demonstrates various XUL elements.
     */
    launchXULTable: function() {
      window.open("chrome://sbdevtools/content/periodic-table/top.xul", 
                  "xulperiodictable", 
                  "chrome,extrachrome,menubar,resizable,width=700,height=550,scrollbars,status,toolbar,titlebar");
    },  
    

    /** 
     * Launch the bugzilla homepage in the browser.
     */
    launchBugzilla: function() {
      var pref = Application.prefs.get("songbird.url.bugzilla");
      if (pref && pref.value) {
        gBrowser.loadURI(pref.value);
      }    
    }, 
    

    /** 
     * Relaunch Songbird
     */
    restart: function() {
      SBDevTools.RestartHelper.restart();
    },   
    
    
    /** 
     * Change the visibility of the bug 
     */
    setUpBugButton: function() {
     
      var toolbar = document.getElementById("control_pane_toolbar");
      if (toolbar && !Application.prefs.get("sbdevtools.bugbutton.addedtodefaults").value) {
        Application.prefs.setValue("sbdevtools.bugbutton.addedtodefaults", true);
        this.updateToolbarItem("control_pane_toolbar", "sbdevtools-bug-toolbarbutton", true);
      } 
    },  
    
    
    /** 
     * Set whether the profile manager should be shown on startup.
     * We can't just restart immediately as nsIAppStartup.eRestart
     * does not reinit the profile :(
     */
    setShowProfileManager: function(show) {
      var profileService = Components.classes["@mozilla.org/toolkit/profile-service;1"]
                                     .getService(Components.interfaces.nsIToolkitProfileService);
                                     
      // In order for the profile manager to show there must be
      // at least 2 profiles
      if (show && profileService.profileCount < 2) {
        profileService.createProfile(null, null, "Testing");
      }
                                     
      profileService.startWithLastProfile = !show;
      profileService.flush();
      
      if (show) {
        alert("The Profile Manager will be shown next time you start Songbird");
      }
    },
    
    
    /** 
     * Enable/disable the XUL cache.  Needed since the cache interferes
     * with RefreshUI
     */
    setXULCacheDisabled: function(disable) {
       Application.prefs.setValue("nglayout.debug.disable_xul_cache",
                                  !!disable); 
    },
    
    
    /**
     * Show/hide a toolbaritem within the given toolbar
     */
    updateToolbarItem: function(toolbarID, itemID, enable) {

      try {
        var toolbar = document.getElementById(toolbarID);
        var curSet = toolbar.currentSet || "";
        
        var hasItem = curSet.indexOf(itemID) != -1;
        
        if (hasItem != enable)
        {
          var set;

          if (hasItem && !enable) {
            set = curSet.replace(itemID, "");
            // Remove comma from end of string if present
            set = set.replace(/,$/,"");            
          } else {
            // Add comma to end of string if not present
            set = curSet.replace(/[^,]$/,",");
            set = toolbar.currentSet + "," + itemID;
          }
          
          toolbar.setAttribute("currentset", set);
          toolbar.currentSet = set;
          document.persist(toolbarID, "currentset");

          // TODO: This is important in FF
          // If you don't do the following call, funny things happen
          //try {
           //BrowserToolboxCustomizeDone(true);
          //}
          //catch (e) { }
        }
      }
      catch(e) { Components.utils.reportError(e.toString()) }      
    } 
  }

  window.addEventListener("load", function() { SBDevTools.Controller.init(); }, false);

}