/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set sw=2 :miv */
/*
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// For information about the licensing and copyright of this Add-On please
// contact POTI, Inc. at customer@songbirdnest.com.
//
*/

/** 
* \file  IPodDeviceServicePaneModule.js
* \brief Javascript source for the iPod device service pane module.
*/

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//
// iPod device service pane module services.
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// iPod device service pane module imported services.
//
//------------------------------------------------------------------------------

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://app/jsmodules/sbProperties.jsm");
Components.utils.import("resource://app/jsmodules/sbStorageFormatter.jsm");


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//
// iPod device service pane module defs.
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

const TYPE_X_SB_TRANSFER_MEDIA_LIST = "application/x-sb-transfer-media-list";
const IPODSP_NS = "http://songbirdnest.com/rdf/ipod-servicepane#";


//------------------------------------------------------------------------------
//
// iPod device service pane module services configuration.
//
//------------------------------------------------------------------------------

//
// className                    Name of component class.
// cid                          Component CID.
// contractID                   Component contract ID.
// ifList                       List of external component interfaces.
// categoryList                 List of component categories.
//
// devCatName                   Name of device category.
// localeBundlePath             Path to locale string bundle.
//
// devURNPrefix                 Device URN prefix.
// libURNPrefix                 Library URN prefix.
// itemURNPrefix                Item URN prefix.
//
// devMgrReadyTopic             Device manager ready event observer topic.
// appQuitTopic                 Application quit event observer topic.
//
// devImgURL                    URL of device service pane node image.
// devBusyImgURL                URL of busy device service pane node image.
// devMgrURL                    URL of device manager window.
// uiElementsDocURL             URL of UI elements document.
//

var IPD_SPMCfg= {
  className: "Songbird iPod Device Service Pane Module",
  cid: Components.ID("{EC76F798-AB27-4B85-A2AD-0D794E0232F9}"),
  contractID: "@songbirdnest.com/servicepane/IPodDevice;1",
  ifList: [ Ci.sbIIPodDeviceServicePaneModule,
            Ci.sbIServicePaneModule,
            Ci.nsIObserver,
            Ci.sbIIPodDeviceCallback,
            Ci.sbIDeviceBaseCallback,
            Ci.sbIMediaListListener ],
  categoryList:
  [
    {
      category: "service-pane",
      entry: "ipod-device"
    }
  ],

  devCatName: "Apple iPod",
  localeBundlePath: "chrome://ipod/locale/IPodDevice.properties",

  devURNPrefix: "urn:device:",
  libURNPrefix: "urn:library:",
  itemURNPrefix: "urn:item:",

  devMgrReadyTopic: "songbird-device-manager-ready",
  appQuitTopic: "quit-application",

  devImgURL: "chrome://ipod/skin/icon_ipod_16x16.png",
  devBusyImgURL: "chrome://songbird/skin/icons/icon-busy.png",
  devMgrURL: "chrome://ipod/content/xul/Manager.xul",
  uiElementsDocURL: "chrome://ipod/content/xul/UIElements.xul"
};


//------------------------------------------------------------------------------
//
// iPod device service pane module services.
//
//------------------------------------------------------------------------------

// IPD_SPM
/**
 * \brief Construct an iPod device service pane module object.
 */

function IPD_SPM()
{
}

/* Set the constructor. */
IPD_SPM.prototype.constructor = IPD_SPM;

IPD_SPM.prototype = {

  //
  // iPod device service pane module services fields.
  //
  //   classDescription         Description of component class.
  //   classID                  Component class ID.
  //   contractID               Component contract ID.
  //   _xpcom_categories        List of component categories.
  //
  //   _cfg                     Configuration settings.
  //   _locale                  Locale string bundle.
  //   _observerSvc             Observer service object.
  //   _devMgr                  Device manager sbIDeviceManager object.
  //   _devCatMgr               Device category manager sbIIPodDevice object.
  //   _devMgrReadyObserverAdded  True if a device manager ready observer has
  //                              been added.
  //   _servicePaneSvc          Service pane service object.
  //   _devServicePaneSvc       Device service pane service object.
  //   _libServicePaneSvc       Library service pane service object.
  //   _devInfoList             List of device information.
  //   _devLibMap               Map of device libraries.
  //   _devLibURNMap            Map of device library service pane node URN's.
  //

  classDescription: IPD_SPMCfg.className,
  classID: IPD_SPMCfg.cid,
  contractID: IPD_SPMCfg.contractID,
  _xpcom_categories: IPD_SPMCfg.categoryList,

  _cfg: IPD_SPMCfg,
  _locale: null,
  _observerSvc: null,
  _devMgr: null,
  _devCatMgr: null,
  _devMgrReadyObserverAdded: false,
  _servicePaneSvc: null,
  _devServicePaneSvc: null,
  _libServicePaneSvc: null,
  _devInfoList: null,
  _devLibMap: null,
  _devLibURNMap: null,


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module sbIIPodDeviceServicePaneModule services.
  //
  //----------------------------------------------------------------------------

  /**
   * \brief Return the iPod device service pane node for the iPod device
   *        specified by aDeviceID.
   *
   * \param aDeviceID           Device identifier.
   *
   * \return iPod device service pane node.
   */

  getDeviceNode: function IPD_SPM_getDeviceNode(aDeviceID) {
    var devInfo = this._devInfoList[aDeviceID];
    if (!devInfo || !devInfo.svcPaneNode)
      throw Cr.NS_ERROR_NOT_AVAILABLE;

    return devInfo.svcPaneNode;
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module sbIServicePaneModule services.
  //
  //----------------------------------------------------------------------------

  /* \brief Initialize this service pane module
   * This is where the module should set itself up in the tree if it hasn't
   * before.
   * \param aServicePaneService the service pane service instance
   */

  servicePaneInit: function IPD_SPM_servicePaneInit(aServicePaneService) {
    /* Save the service pane service. */
    this._servicePaneSvc = aServicePaneService;

    /* Initialize the services. */
    this._initialize();
  },


  /* \brief Fill the context menu for the given node
   * \param aNode the node that was context-clicked on
   * \param aContextMenu the menu we're trying to fill
   * \param aParentWindow the toplevel window we're displaying in
   */

  fillContextMenu: function IPD_SPM_fillContextMenu(aNode,
                                                    aContextMenu,
                                                    aParentWindow) {
    try { this._fillContextMenu(aNode, aContextMenu, aParentWindow); }
    catch (ex) { dump("fillContextMenu exception: " + ex + "\n"); }
  },

  _fillContextMenu: function IPD_SPM__fillContextMenu(aNode,
                                                      aContextMenu,
                                                      aParentWindow) {
    // Get the device ID.  Do nothing if not a device node.
    var deviceID = aNode.getAttributeNS(IPODSP_NS, "device-id");
    if (!deviceID)
      return;

    // Add the device context menu.
    this._addUIElements(aContextMenu,
                        "ipod.device.context.menu",
                        { dev_id: deviceID });
    var menuDoc = aContextMenu.ownerDocument;

    // Get the device configuration and state.
    var mgmtType = this._devCatMgr.getMgmtType(deviceID);
    var state = this._devCatMgr.STATE_IDLE;
    try {
        state = this._devCatMgr.getDeviceState(deviceID);
    } catch (ex) { }

    // Set the device name menu item.
    var devName = IPString.get("ipod.device_name", "Apple iPod", this._locale) +
                  " (" + IPDUtils.getModelSize(deviceID) + ")";
    var deviceNameMI =
          menuDoc.getElementById("ipod.device.context.device_name");
    deviceNameMI.setAttribute("label", devName);

    // Hide "sync" menu item, show "cancel sync" menu item, and disable the
    // "eject" menu item if device is not idle.
    if (state != this._devCatMgr.STATE_IDLE) {
      var syncMI = menuDoc.getElementById("ipod.device.context.sync");
      var cancelSyncMI =
            menuDoc.getElementById("ipod.device.context.cancel_sync");
      syncMI.hidden = true;
      cancelSyncMI.hidden = false;
      var ejectMI = menuDoc.getElementById("ipod.device.context.eject");
      ejectMI.setAttribute("disabled", "true");
    }

    // Disable the "new playlist" menu item if not in manual management mode.
    if (mgmtType != Ci.sbIIPodDevice.MGMT_TYPE_MANUAL) {
      var newPlaylistMI =
            menuDoc.getElementById("ipod.device.context.new_playlist");
      newPlaylistMI.setAttribute("disabled", "true");
    }
  },

  
  /* \brief Fill a menu with menuitems to create new tree nodes
   * \param aNode the node or null for the click and hold menu
   * \param aContextMenu the menu we're trying to fill
   * \param aParentWindow the toplevel window we're displaying in
   */

  fillNewItemMenu: function IPD_SPM_fillNewItemMenu(aNode,
                                                    aContextMenu,
                                                    aParentWindow) { },


  /* \brief Selection changes notification
   * \param aNode the node that is now selected
   * \param aParentWindow the toplevel window we're displaying in
   */

  onSelectionChanged: function IPD_SPM_onSelectionChanged(aNode,
                                                          aContainer,
                                                          aParentWindow) { },


  /* \brief Return whether the item can be dropped on the node
   */

  canDrop: function IPD_SPM_canDrop(aNode,
                                    aDragSession,
                                    aOrientation,
                                    aWindow) {
    // Don't accept any drops that are not on the node
    if (aOrientation != Ci.nsITreeView.DROP_ON) {
      return false;
    }

    // Get the device ID.
    var deviceId = aNode.getAttributeNS(IPODSP_NS, "device-id");
    if (!deviceId)
      return;

    // Get the management type and determine if the item is a list.
    var mgmtType = this._devCatMgr.getMgmtType(deviceId);
    var isList =
          aDragSession.isDataFlavorSupported(TYPE_X_SB_TRANSFER_MEDIA_LIST);

    // Lists can be dropped if the device is in manual management mode.
    if (isList && (mgmtType == Ci.sbIIPodDevice.MGMT_TYPE_MANUAL)) {
      var context = this._getDndData(aDragSession,
                                     TYPE_X_SB_TRANSFER_MEDIA_LIST,
                                     Ci.sbISingleListTransferContext);
      var list = context.list;

      // Don't allow a drop if this list is from the ipod itself
      if (list.library.equals(this._devInfoList[deviceId].devLib)) {
        return false;
      }

      return true;
    }

    return false;
  },


  /* \brief Handle dropping of an item on node
   */

  onDrop: function IPD_SPM_onDrop(aNode, aDragSession, aOrientation, aWindow) {
    // Get the device ID.
    var deviceId = aNode.getAttributeNS(IPODSP_NS, "device-id");
    if (!deviceId)
      return;

    // Get the management type and determine if the item is a list.
    var mgmtType = this._devCatMgr.getMgmtType(deviceId);
    var isList =
          aDragSession.isDataFlavorSupported(TYPE_X_SB_TRANSFER_MEDIA_LIST);

    // If the item is a list and the managmenet type is manual, copy the list to
    // the device.
    if (isList && (mgmtType == Ci.sbIIPodDevice.MGMT_TYPE_MANUAL)) {
      var context = this._getDndData(aDragSession,
                                     TYPE_X_SB_TRANSFER_MEDIA_LIST,
                                     Ci.sbISingleListTransferContext);
      this._devInfoList[deviceId].devLib.copyMediaList("simple", context.list);
    }
  },


  /* \brief Handle drag gesture
   */

  onDragGesture: function IPD_SPM_onDragGesture(aNode, aTransferable) { },


  /* \brief Called when a node is renamed by the user, allows
   *        the module to accept the action by setting the
   *        node name to the given value.
   * \param aNode the node that was renamed
   * \param aNewName new name entered by the user
   */

  onRename: function IPD_SPM_onRename(aNode, aNewName) {
    // Get the device ID.  Do nothing if not a device node.
    var deviceID = aNode.getAttributeNS(IPODSP_NS, "device-id");
    if (!deviceID)
      return;

    // Set the device user set name.
    this._devCatMgr.setUserSetName(deviceID, aNewName);
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module nsIObserver services.
  //
  //----------------------------------------------------------------------------
  
  /**
   * \brief Handle the observed event specified by aSubject, aTopic, and aData.
   *
   * \param aSubject            Event subject.
   * \param aTopic              Event topic.
   * \param aData               Event data.
   */

  observe: function IPD_SPM_observe(aSubject, aTopic, aData) {
    try { this._observe(aSubject, aTopic, aData); }
    catch (ex) { dump("observe exception: " + ex + "\n"); }
  },

  _observe: function IPD_SPM__observe(aSubject, aTopic, aData) {
    // Dispatch processing of the event.
    switch (aTopic) {
      case this._cfg.devMgrReadyTopic :
        this._handleDevMgrReady();
        break;

      case this._cfg.appQuitTopic :
        this._handleAppQuit();
        break;
    }
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module sbIDeviceBaseCallback and
  // sbIIPodDeviceCallback services.
  //
  //----------------------------------------------------------------------------

  /**
   * \brief Handle the device connected event for the device specified by
   *        aDeviceIdentifier.
   *
   * \param aDeviceIdentifier   The device unique identifier.
   */

  onDeviceConnect: function IPD_SPM_onDeviceConnect(aDeviceIdentifier) {
    try { this._onDeviceConnect(aDeviceIdentifier); }
    catch (ex) { dump("onDeviceConnect exception: " + ex + "\n"); }
  },

  _onDeviceConnect: function IPD_SPM__onDeviceConnect(aDeviceIdentifier) {
    // Add the device to the service pane.
    this._addDevice(aDeviceIdentifier);
  },


  /**
   * \brief Handle the device disconnected event for the device specified by
   *        aDeviceIdentifier.
   *
   * \param aDeviceIdentifier   The device unique identifier.
   */

  onDeviceDisconnect: function IPD_SPM_onDeviceDisconnect(aDeviceIdentifier) {
    try { this._onDeviceDisconnect(aDeviceIdentifier); }
    catch (ex) { dump("onDeviceDisconnect exception: " + ex + "\n"); }
  },

  _onDeviceDisconnect: function IPD_SPM__onDeviceDisconnect(aDeviceIdentifier) {
    // Remove the device from the service pane.
    this._removeDevice(aDeviceIdentifier);
  },


  /**
   * \brief Handle the device state changed event for the device specified by
   *        aDeviceIdentifier.  The new device state is specified by aState.
   *
   * \param aDeviceIdentifier   The device unique identifier.
   * \param aState              The device state.
   */

  onStateChanged: function IPD_SPM_onStateChanged(aDeviceIdentifier, aState) {
    // Update the device state.
    this._updateDeviceState(aDeviceIdentifier);
  },


  /**
   * \brief Handle device mount completion callbacks.  These callbacks are
   * delivered when the device has been fully mounted and its associated media
   * library is complete and usable.
   * onDeviceMountComplete
   *
   * \param aDeviceIdentifier The device unique identifier.
   */

  onDeviceMountComplete:
    function IPD_SPM_onDeviceMountComplete(aDeviceIdentifier) {
  },


  /**
   * \brief Handle FairPlay not authorized callbacks.  The FairPlay account
   * information is specified by aUserID, aAccountName, and aUserName.
   *
   * \param aDeviceIdentifier The device unique identifier.
   * \param aUserID FairPlay user ID.
   * \param aAccountName FairPlay account name.
   * \param aUserName FairPlay user name.
   */

  onDeviceFPNotAuthorized:
    function IPD_SPM_onDeviceFPNotAuthorized(aDeviceIdentifier,
                                             aUserID,
                                             aAccountName,
                                             aUserName) {
  },


  /**
   * \brief Handle changes to the device preferences.
   *
   * \param aDeviceIdentifier The device unique identifier.
   */

  onDevicePrefsChanged:
    function IPD_SPM_onDevicePrefsChanged(aDeviceIdentifier) {
    // Update the read-only property for all device nodes.
    this._setAllNodesReadOnly(aDeviceIdentifier);
  },


  /**
   * \brief Handle changes to the device user set name.
   *
   * \param aDeviceIdentifier The device unique identifier.
   */

  onDeviceUserSetNameChanged:
    function IPD_SPM_onDeviceUserSetNameChanged(aDeviceIdentifier) {
    // Get the device info.
    var devInfo = this._devInfoList[aDeviceIdentifier];
    if (!devInfo)
      return;

    // Set the device service pane node name.
    devInfo.svcPaneNode.name = this._devCatMgr
                                   .getUserSetName(aDeviceIdentifier);
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module sbIMediaListListener services.
  //
  //----------------------------------------------------------------------------

  //
  // _inBatchMap                Map of media lists in a batch update.
  // _rescanListMap             Map of media lists to be rescanned.
  //

  _inBatchMap: null,
  _rescanListMap: null,


  /**
   * \brief Initialize the sbIMediaListListener services.
   */

  _SBIMLL_Initialize: function IPD_SPM__SBIMLL_Initialize() {
    this._inBatchMap = { };
    this._rescanListMap = { };
  },


  /**
   * \brief Finalize the sbIMediaListListener services.
   */

  _SBIMLL_Finalize: function IPD_SPM__SBIMLL_Finalize() {
    this._inBatchMap = null;
    this._rescanListMap = null;
  },


  /**
   * \brief Called when a media item is added to the list.
   * \param aMediaList The list that has changed.
   * \param aMediaItem The new media item.
   * \return True if you do not want any further onItemAdded notifications for
   *         the current batch.  If there is no current batch, the return value
   *         is ignored.
   */

  onItemAdded: function IPD_SPM_onItemAdded(aMediaList, aMediaItem, aIndex) {
    var returnVal = false;
    try { returnVal = this._onItemAdded(aMediaList, aMediaItem); }
    catch (ex) { dump("onItemAdded exception: " + ex + "\n"); }
    return (returnVal);
  },

  _onItemAdded: function IPD_SPM__onItemAdded(aMediaList, aMediaItem) {
    var                         noFurtherNotifications = false;

    // If in a batch, mark media list for rescanning.  Otherwise, add any
    // playlists to the device playlist maps.
    if (this._inBatchMap[aMediaList.guid]) {
      this._rescanListMap[aMediaList.guid] = true;
      noFurtherNotifications = true;
    } else {
      if (aMediaItem instanceof Components.interfaces.sbIMediaList)
        this._addDevPlaylist(aMediaItem);
    }

    return noFurtherNotifications;
  },


  /**
   * \brief Called before a media item is removed from the list.
   * \param aMediaList The list that has changed.
   * \param aMediaItem The media item to be removed
   * \return True if you do not want any further onBeforeItemRemoved
   *         notifications for the current batch.  If there is no current batch,
   *         the return value is ignored.
   */

  onBeforeItemRemoved: function IPD_SPM_onBeforeItemRemoved(aMediaList,
                                                            aMediaItem,
                                                            aIndex) {
    var                         noFurtherNotifications = false;

    // If in a batch, mark media list for rescanning.  Otherwise, remove any
    // playlists from the device playlist maps.
    if (this._inBatchMap[aMediaList.guid]) {
      this._rescanListMap[aMediaList.guid] = true;
      noFurtherNotifications = true;
    } else {
      if (aMediaItem instanceof Components.interfaces.sbIMediaList)
        this._removeDevPlaylist(aMediaItem);
    }

    return noFurtherNotifications;
  },


  /**
   * \brief Called after a media item is removed from the list.
   * \param aMediaList The list that has changed.
   * \param aMediaItem The removed media item.
   * \return True if you do not want any further onAfterItemRemoved for the
   *         current batch.  If there is no current batch, the return value is
   *         ignored.
   */

  onAfterItemRemoved: function IPD_SPM_onAfterItemRemoved(aMediaList,
                                                          aMediaItem,
                                                          aIndex) {
    return true;
  },


  /**
   * \brief Called when a media item is changed.
   * \param aMediaList The list that has changed.
   * \param aMediaItem The item that has changed.
   * \param aProperties Array of properties that were updated.  Each property's
   *        value is the previous value of the property
   * \return True if you do not want any further onItemUpdated notifications
   *         for the current batch.  If there is no current batch, the return
   *         value is ignored.
   */

  onItemUpdated: function IPD_SPM_onItemUpdated(aMediaList,
                                                aMediaItem,
                                                aIndex,
                                                aProperties) {
    return true;
  },


  /**
   * \brief Called when a media list is cleared.
   * \return True if you do not want any further onListCleared notifications
   *         for the current batch.  If there is no current batch, the return
   *         value is ignored.
   */

  onListCleared: function IPD_SPM_onListCleared(aMediaList) {
    var                         noFurtherNotifications = false;

    // If in a batch, mark media list for rescanning.  Otherwise, update the
    // device playlist maps.
    if (this._inBatchMap[aMediaList.guid]) {
        this._rescanListMap[aMediaList.guid] = true;
        noFurtherNotifications = true;
    } else {
        this._updateDevPlaylists(aMediaList);
    }

    return noFurtherNotifications;
  },


  /**
   * \brief Called when the library is about to perform multiple operations at
   *        once.
   *
   * This notification can be used to optimize behavior. The consumer may
   * choose to ignore further onItemAdded or onItemRemoved notifications until
   * the onBatchEnd notification is received.
   *
   * \param aMediaList The list that has changed.
   */

  onBatchBegin: function IPD_SPM_onBatchBegin(aMediaList) {
    // Add media list to the in batch map.
    this._inBatchMap[aMediaList.guid] = true;
  },


  /**
   * \brief Called when the library has finished performing multiple operations
   *        at once.
   *
   * This notification can be used to optimize behavior. The consumer may
   * choose to stop ignoring onItemAdded or onItemRemoved notifications after
   * receiving this notification.
   *
   * \param aMediaList The list that has changed.
   */

  onBatchEnd: function(aMediaList) {
    // Remove media list from the in batch map.
    delete this._inBatchMap[aMediaList.guid];

    // Update device playlists if needed.
    if (this._rescanListMap[aMediaList.guid]) {
      this._updateDevPlaylists(aMediaList);
      delete(this._rescanListMap[aMediaList.guid]);
    }
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module nsISupports services.
  //
  //----------------------------------------------------------------------------

  QueryInterface: XPCOMUtils.generateQI(IPD_SPMCfg.ifList),


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module event handling services.
  //
  //----------------------------------------------------------------------------

  /**
   * \brief Handle device manager ready events.
   */

  _handleDevMgrReady: function IPD_SPM__handleDevMgrReady() {
    // Initialize the device category manager services.
    this._devCatInitialize();
  },


  /**
   * \brief Handle application quit events.
   */

  _handleAppQuit: function IPD_SPM__handleAppQuit() {
    // Finalize the services.
    this._finalize();
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module UI services.
  //
  //----------------------------------------------------------------------------

  //
  // _uiElementsDoc             Document containing UI elements.
  //

  _uiElementsDoc: null,


  /**
   * \brief Add the UI element child nodes of the node specified by
   *        aSrcNodeListID to the target UI element node specified by aTgtNode.
   *        For each node added, set each attribute contained in the attribute
   *        object specified by aAttrList.
   *
   * \param aTgtNode            Target node to which to add.
   * \param aSrcNodeListID      ID of source node list.
   * \param aAttrList           List of attributes to assign to added nodes.
   */

  _addUIElements: function IPD_SPM__addUIElements(aTgtNode,
                                                  aSrcNodeListID,
                                                  aAttrList) {
    // Load the UI elements document.
    this._loadUIElementsDoc();

    // Get the target document and source node list.
    var tgtDoc = aTgtNode.ownerDocument;
    var srcNodeList = this._uiElementsDoc.getElementById(aSrcNodeListID)
                                         .childNodes;

    // Add the source nodes.
    for (var i = 0; i < srcNodeList.length; i++) {
      // Get the next source node.  Skip if not a XUL element (e.g., text node).
      var srcNode = srcNodeList[i];
      if (!(srcNode instanceof Components.interfaces.nsIDOMXULElement))
        continue;

      // Import source node into the target document.
      srcNode = tgtDoc.importNode(srcNode, true);

      // Add the attributes.
      for (var attrName in aAttrList) {
          srcNode.setAttribute(attrName, aAttrList[attrName]);
      }

      // Add the source node to the target node.
      aTgtNode.appendChild(srcNode);
    }
  },


  /**
   * \brief Load the document containing UI elements.
   */

  _loadUIElementsDoc: function IPD_SPM__loadUIElementsDoc() {
    // Do nothinf if document already loaded.
    if (this._uiElementsDoc)
      return;

    // Load the document.
    var ioSvc = Cc["@mozilla.org/network/io-service;1"].getService
                                                          (Ci.nsIIOService);
    var channel = ioSvc.newChannel(this._cfg.uiElementsDocURL, null, null);
    var inputStream = channel.open();
    var domParser = Cc["@mozilla.org/xmlextras/domparser;1"]
                      .createInstance(Ci.nsIDOMParser);
    this._uiElementsDoc = domParser.parseFromStream(inputStream,
                                                    null,
                                                    channel.contentLength,
                                                    "text/xml");
  },


  //----------------------------------------------------------------------------
  //
  // iPod device service pane module services.
  //
  //----------------------------------------------------------------------------

  /**
   * \brief Initialize the iPod device service pane module.
   */

  _initialize: function IPD_SPM__initialize() {
    // Initialize the device lists and maps.
    this._devInfoList = {};
    this._devLibMap = {};
    this._devLibURNMap = {};

    // Get the locale string bundle.
    var stringBundleSvc = Cc["@mozilla.org/intl/stringbundle;1"]
                            .getService(Ci.nsIStringBundleService);
    this._locale = stringBundleSvc.createBundle(this._cfg.localeBundlePath);

    // Set up observers.
    this._observerSvc = Cc["@mozilla.org/observer-service;1"]
                          .getService(Ci.nsIObserverService);
    this._observerSvc.addObserver(this, this._cfg.appQuitTopic, false);

    // Remove all device nodes.
    this._removeDevNodes(this._servicePaneSvc.root);

    // Get the device manager and device and library service pane services.
    this._devMgr = Cc["@songbirdnest.com/Songbird/DeviceManager;1"]
                     .getService(Ci.sbIDeviceManager);
    this._devServicePaneSvc = Cc["@songbirdnest.com/servicepane/device;1"]
                                .getService(Ci.sbIDeviceServicePaneService);
    this._libServicePaneSvc = Cc["@songbirdnest.com/servicepane/library;1"]
                                .getService(Ci.sbILibraryServicePaneService);

    // Initialize the sbIMediaListListener services.
    this._SBIMLL_Initialize();

    // Initialize the device category manager services.
    this._devCatInitialize();
  },


  /**
   * \brief Finalize the iPod device service pane module.
   */

  _finalize: function IPD_SPM__finalize() {
    // Remove observers.
    this._observerSvc.removeObserver(this, this._cfg.appQuitTopic);
    if (this._devMgrReadyObserverAdded) {
      this._observerSvc.removeObserver(this, this._cfg.devMgrReadyTopic);
      this._devMgrReadyObserverAdded = false;
    }

    // Finalize the iPod device utilities.
    if (typeof(IPDUtils) != "undefined")
        IPDUtils.finalize();

    // Remove the device callback.
    if (this._devCatMgr)
      this._devCatMgr.removeCallback(this);

    // Remove all devices.
    this._removeAllDevices();

    // Remove all device nodes.
    this._removeDevNodes(this._servicePaneSvc.root);

    // Finalize the sbIMediaListListener services.
    this._SBIMLL_Finalize();

    // Clear the device lists and maps.
    this._devInfoList = null;
    this._devLibMap = null;
    this._devLibURNMap = null;

    // Clear object references.
    this._observerSvc = null;
    this._devMgr = null;
    this._servicePaneSvc = null;
  },


  /**
   * \brief Initialize the iPod device service pane module device category
   *        services.
   */

  _devCatInitialize: function IPD_SPM__devCatInitialize() {
    // Do nothing if device category manager is set up.
    if (this._devCatMgr)
      return;

    // Check if the device category is available.  If an exception is thrown,
    // the device manager is not ready.  Wait for it to be ready.
    var hasDevCat = false;
    try {
      hasDevCat = this._devMgr.hasDeviceForCategory(this._cfg.devCatName);
    } catch (ex) {
      if (!this._devMgrReadyObserverAdded) {
        this._observerSvc.addObserver(this,
                                      this._cfg.devMgrReadyTopic,
                                      false);
        this._devMgrReadyObserverAdded = true;
      }
      return;
    }

    // Do nothing more if device category is not available.
    if (!hasDevCat)
      return;

    // Get the device category manager.
    this._devCatMgr = this._devMgr.getDeviceByCategory(this._cfg.devCatName);
    this._devCatMgr = this._devCatMgr.QueryInterface(Ci.sbIIPodDevice);

    // Add device callbacks and deliver callbacks for already connected devices.
    this._devCatMgr.addCallback(this);
    this._devCatMgr.deliverOnDeviceConnectCallbacks(this);

    // Get the iPod device utilities.
    var jsLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                             .getService
                                (Components.interfaces.mozIJSSubScriptLoader);
    jsLoader.loadSubScript("chrome://ipod/content/scripts/IPodDeviceUtils.js",
                           null);

    // Remove device manager ready observer.
    if (this._devMgrReadyObserverAdded) {
        this._observerSvc.removeObserver(this, this._cfg.devMgrReadyTopic);
        this._devMgrReadyObserverAdded = false;
    }
  },


  /**
   * \brief Add the device specified by aDeviceID to the service pane.
   *
   * \param aDeviceID           The device unique identifier.
   */

  _addDevice: function IPD_SPM__addDevice(aDeviceID) {
    // Do nothing if device has already been added.
    if (this._devInfoList[aDeviceID])
      return;

    // Create a device info list entry.
    this._devInfoList[aDeviceID] = {};
    this._devInfoList[aDeviceID].playlistList = {};

    // Add a device node in the service pane.
    var devNode = this._devServicePaneSvc.createNodeForDevice(this._devCatMgr,
                                                              aDeviceID);
    devNode.setAttributeNS(IPODSP_NS, "device-id", aDeviceID);
    devNode.contractid = this._cfg.contractID;
    devNode.name = this._devCatMgr.getUserSetName(aDeviceID);
    devNode.image = this._cfg.devImgURL;
    devNode.hidden = false;
    devNode.url = this._cfg.devMgrURL + "?deviceIdentifier=" + aDeviceID;
    devNode.editable = true;
    this._devInfoList[aDeviceID].svcPaneNode = devNode;

    // Add the device library.
    var devLib = this._devCatMgr.getLibrary(aDeviceID);
    if (devLib)
      this._addDevLib(devLib, aDeviceID);

    // Update the device playlists.
    if (devLib)
      this._updateDevPlaylists(devLib);

    // Update the device state.
    this._updateDeviceState(aDeviceID);
  },


  /**
   * \brief Remove the device specified by aDeviceID from the service pane.
   *
   * \param aDeviceID           The device unique identifier.
   */

  _removeDevice: function IPD_SPM__removeDevice(aDeviceID) {
    // Get the device info.  Do nothing if no device info available.
    var devInfo = this._devInfoList[aDeviceID];
    if (!devInfo)
      return;

    // Remove the device library.
    var devLib = devInfo.devLib;
    if (devLib)
      this._removeDevLib(devLib);

    // Remove the device node.
    this._servicePaneSvc.removeNode(devInfo.svcPaneNode);

    // Remove device info list entry.
    delete this._devInfoList[aDeviceID];
  },


  /**
   * \brief Remove all devices from the service pane.
   */

  _removeAllDevices: function IPD_SPM__removeAllDevices() {
    // Remove all devices.
    for (var deviceID in this._devInfoList)
      this._removeDevice(deviceID);
  },


  /**
   * \brief Remove all device nodes from the service pane node specified by
   *        aNode.  If aNode or any of its descendents are device nodes, they
   *        are removed.
   *
   * \param aNode               Node from which to remove device nodes.
   */

  _removeDevNodes: function IPD_SPM__removeDevNodes(aNode) {
    // Remove child device nodes.
    if (aNode.isContainer) {
      var childEnum = aNode.childNodes;
      while (childEnum.hasMoreElements()) {
        var child = childEnum.getNext().QueryInterface(Ci.sbIServicePaneNode);
        this._removeDevNodes(child);
      }
    }

    // Remove device nodes.
    if (aNode.contractid == this._cfg.contractID)
      this._servicePaneSvc.removeNode(aNode);
  },


  /**
   * \brief Add the device library specified by aLibrary for the device
   *        specified by aDeviceID to the list of device libraries.
   *
   * \param aLibrary            Device library to add.
   * \param aDeviceID           Device identifier.
   */

  _addDevLib: function IPD_SPM__addDevLib(aLibrary, aDeviceID) {
    // Add the device library to the device library map.
    this._devLibMap[aLibrary.guid] = aDeviceID;
    this._devLibURNMap[this._cfg.libURNPrefix + aLibrary.guid] = aDeviceID;
    this._devInfoList[aDeviceID].devLib = aLibrary;

    // Create the device library service pane node and move it underneath the
    // device service pane node.
    var devLibNode = this._libServicePaneSvc.createNodeForLibrary(aLibrary);
    this._devInfoList[aDeviceID].svcPaneNode.appendChild(devLibNode);
    this._devInfoList[aDeviceID].devLibNode = devLibNode;

    // Set the read-only property for the device library node.
    this._setNodeReadOnly(aDeviceID, devLibNode);

    // Add a listener for the device library. */
    aLibrary.addListener(this,
                         false,
                         Ci.sbIMediaList.LISTENER_FLAGS_ALL &
                           ~Ci.sbIMediaList.LISTENER_FLAGS_ITEMUPDATED);
  },


  /**
   * \brief Remove the device library specified by aLibrary from the list of
   *        device libraries.
   *
   * \param aLibrary            Device library to remove.
   */

  _removeDevLib: function IPD_SPM__removeDevLib(aLibrary) {
    // Remove the device library from the device library map.
    delete (this._devLibMap[aLibrary.guid]);
    delete (this._devLibURNMap[this._cfg.libURNPrefix + aLibrary.guid]);

    // Remove the device library listener.
    aLibrary.removeListener(this);
  },


  /**
   * \brief Add the device playlist specified by aMediaList to the list of
   *        device playlists.
   *
   * \param aMediaList          Device playlist to add.
   */

  _addDevPlaylist: function IPD_SPM__addDevPlaylist(aMediaList) {
    // Get the media list device ID and playlist URN.
    var deviceID = this._devLibMap[aMediaList.library.guid];
    var playlistURN = this._cfg.itemURNPrefix + aMediaList.guid;

    // Get the device info record.
    var devInfo = this._devInfoList[deviceID];

    // Do nothing if playlist already added.
    if (devInfo.playlistList[playlistURN])
      return;

    // Get the playlist service pane node, creating one if necessary.
    var playlistNode = this._libServicePaneSvc
                           .getNodeForLibraryResource(aMediaList);
    if (!playlistNode) {
      playlistNode = this._servicePaneSvc.addNode(playlistURN,
                                                  this._servicePaneSvc.root,
                                                  true);
    }

    // Set the read-only property for the playlist node.
    this._setNodeReadOnly(deviceID, playlistNode);

    // Move the playlist service pane node underneath the device service pane
    // node.
    devInfo.svcPaneNode.appendChild(playlistNode);

    // Add the playlist to the device playlist list.
    devInfo.playlistList[playlistURN] = playlistNode;
  },


  /**
   * \brief Remove the device playlist specified by aMediaList from the list of
   *        device playlists.
   *
   * \param aMediaList          Device playlist to remove.
   */

  _removeDevPlaylist: function IPD_SPM__removeDevPlaylist(aMediaList) {
    // Remove the playlist from the device playlist list.
    var deviceID = this._devLibMap[aMediaList.library.guid];
    var playlistURN = this._cfg.itemURNPrefix + aMediaList.guid;
    delete this._devInfoList[deviceID].playlistList[playlistURN];
  },


  /**
   * \brief Update the list of device playlists with the playlists contained in
   *        the media list specified by aMediaList.
   *
   * \param aMediaList          Media list from which to update.
   */

  _updateDevPlaylists: function IPD_SPM__updateDevPlaylists(aMediaList) {
    // Set up an enumeration listener to collect items into a simple array.
    var enumListener = {
      itemList: [],
      onEnumerationBegin: function() { },
      onEnumerationEnd: function() { },
      onEnumeratedItem: function(aList, aItem) {
        this.itemList.push(aItem);
      }
    };

    // Create a list of media lists.
    aMediaList.enumerateItemsByProperty(SBProperties.isList, "1", enumListener);

    // Add each media list as a device playlist.
    var length = enumListener.itemList.length;
    for (var i = 0; i < length; i++) {
      this._addDevPlaylist(enumListener.itemList[i]);
    }
  },


  /**
   * \brief Update the device state in the service pane for the device specified
   *        by aDeviceID.
   *
   * \param aDeviceID           Device identifier.
   */

  _updateDeviceState: function IPD_SPM__updateDeviceState(aDeviceID) {
    // Get the device service pane node.  Do nothing if not found.
    var devNode = null;
    if (this._devInfoList[aDeviceID])
      devNode = this._devInfoList[aDeviceID].svcPaneNode;
    if (!devNode)
      return;

    // Get the device state.  Default to idle on failure to get state.
    var state = this._devCatMgr.STATE_IDLE;
    try {
        state = this._devCatMgr.getDeviceState(aDeviceID);
    } catch (ex) { }

    // Set the device service pane node image according to the state.
    if (state == this._devCatMgr.STATE_IDLE)
      devNode.image = this._cfg.devImgURL;
    else
      devNode.image = this._cfg.devBusyImgURL;
  },


  /**
   * \brief Get the data from the drag-and-drop session specified by
   *        aDragSession.  The data type to get is specified by aDataType and
   *        the returned data is a component with the interface specified by
   *        aInterface.
   *
   * \param aDragSession        Drag-and-drop session for which to get data.
   * \param aDataType           Type of data to get.
   * \param aInterface          Type of component interface to get.
   *
   * \return Drag-and-drop data component.
   */

  _getDndData: function IPD_SPM__getDndData(aDragSession,
                                            aDataType,
                                            aInterface) {
    // create an nsITransferable
    var transferable = Cc["@mozilla.org/widget/transferable;1"]
                         .createInstance(Ci.nsITransferable);

    // specify what kind of data we want it to contain
    transferable.addDataFlavor(aDataType);

    // ask the drag session to fill the transferable with that data
    aDragSession.getData(transferable, 0);

    // get the data from the transferable
    var data = {};
    var dataLength = {};
    transferable.getTransferData(aDataType, data, dataLength);

    // it's always a string. always.
    data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
    data = data.toString();

    // get the object from the dnd source tracker
    var dnd = Cc["@songbirdnest.com/Songbird/DndSourceTracker;1"]
                .getService(Ci.sbIDndSourceTracker);
    return dnd.getSource(data).QueryInterface(aInterface);
  },


  /**
   * \brief Set the read-only property for all nodes of the device specified by
   *        aDeviceID as appropriate for the current device state.
   *
   * \param aDeviceID           Device for which to set node read-only
   *                            properties.
   */

  _setAllNodesReadOnly: function IPD_SPM__setAllNodesReadOnly(aDeviceID) {
    // Get the device info record.
    var devInfo = this._devInfoList[aDeviceID];

    // Set the read-only property for the device library node.
    this._setNodeReadOnly(aDeviceID, devInfo.devLibNode);

    // Set the read-only property for each device playlist.
    for (var playlistURN in devInfo.playlistList) {
      this._setNodeReadOnly(aDeviceID, devInfo.playlistList[playlistURN]);
    }
  },


  /**
   * \brief Set the read-only property for the node specified by aNode as
   *        appropriate for the current state of the device specified by
   *        aDeviceID.
   *
   * \param aDeviceID           Device for which to set node read-only property.
   * \param aNode               Node for which to set read-only property.
   */

  _setNodeReadOnly: function IPD_SPM__setNodeReadOnly(aDeviceID, aNode) {
    // Set node to read-only if not in manual management mode.
    var mgmtType = this._devCatMgr.getMgmtType(aDeviceID);
    if (mgmtType == Ci.sbIIPodDevice.MGMT_TYPE_MANUAL)
      this._libServicePaneSvc.setNodeReadOnly(aNode, false);
    else
      this._libServicePaneSvc.setNodeReadOnly(aNode, true);
  }
};


//------------------------------------------------------------------------------
//
// iPod device service pane module component services.
//
//------------------------------------------------------------------------------

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([IPD_SPM]);
}

