/*
 * CDDL HEADER START
 *       
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 * 
 * You can obtain a copy of the license in LICENSE.TXT or at
 * http://www.opensource.org/licenses/cddl1.php
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at LICENSE.TXT
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

// Variables for the Flickr stream
const flickrKey = "33a6c9b21ada1e5b7d85f5cde788e6c7";
// Flickr stream timings
const cflickrShift = -150;
const cflickrInterval = 2000;
const sflickrShift = -2;
const sflickrInterval = 50;
var flickrShift;
var flickrInterval;
var flickrIdx;
var smoothScroll = false;
var verticalOrient = false;

var Cc=Components.classes;
var Ci=Components.interfaces;
var gBrowser;
const PREFSERVICE="@mozilla.org/preferences-service;1";
const youTubeLimitPref	= "extensions.mashtape.youtube_amount";
const flickrLimitPref	= "extensions.mashtape.flickr_amount";
const flickrSearchTypePref	= "extensions.mashtape.flickr_search_type";
const flickrScrollPref	= "extensions.mashtape.flickr_smooth_scroll";
const flickrSortPref	= "extensions.mashtape.flickr_sort";

if (typeof mashTape == "undefined") {
	    var mashTape = {};
}

var gBundle = Components.classes["@mozilla.org/intl/stringbundle;1"].
		getService(Components.interfaces.nsIStringBundleService);
var strings = gBundle.
		createBundle("chrome://songbird-mashtape/locale/mashtape.properties");

function mashTape_enableOtherSongsButton() {
	document.getElementById("otherSongs").disabled = false;
}

function mashTape_openURL() {
	tagbox = document.getElementById("tag-box");
	url = tagbox.getSelectedItem(0).getAttribute("value");
	mashTape_openNewLink(url);
}

function mashTape_setEventInfo() {
	document.getElementById("eventInfo").disabled = false;
}

function mashTape_openEventURL() {
	eventbox = document.getElementById("event-box");
	url = eventbox.getSelectedItem(0).getAttribute("href");
	mashTape_openNewLink(url);
}

mashTape.updateInfo = function(topic, data) {
	if (typeof(topic) == "undefined" || typeof(data) == "undefined" ||
		(data == ""))
		return;

    /* kludge to make local copies of the metadata to make sure we've got
	   all the necessary fields before we attempt to do anything with them
	 */
	if (topic == "metadata.artist") {
		this.artist = data;
		if (typeof(this.timerFn) != "undefined")
			clearTimeout(this.timerFn);
	} else if (topic == "metadata.album")
		this.album = data;
	else if (topic == "metadata.title")
		this.track = data;
	
	this.doQueries()
}

mashTape.doQueries = function() {
	/* if artist or track are undefined, then they haven't been set yet, so
	   we're not ready.  album is okay since it could be a single
	 */
	if (typeof(this.artist) == "undefined" || typeof(this.track) == "undefined")
		return;
	
	var artist = this.artist;
	var album = this.album;
	var track = this.track;
	delete this.artist;
	delete this.album;
	delete this.track;
	delete this.eventsFound;

	this.updateEvents(artist);
	this.updateWikipedia(artist);
	this.updateFlickrFeed(artist);
	this.updateYouTube(artist);
	this.updateLyrics(artist, track);
	this.updateTags(artist, album, track);
	this.updateArtistBio(artist);
}

mashTape.lastFMURL = function() {
	var args = Array.prototype.slice.call(arguments);
	var query = escape(args.shift());
	var key = escape(args.shift());
	var val = escape(args.shift());
	if (args.length > 0)
		val = val + "/" + escape(args.shift());

	var url = "http://ws.audioscrobbler.com/1.0/" + key + "/" + val + "/" + 
		query;
	if (query == "events")
		url += ".rss";
	else
		url += ".xml";
	return(url);
}

mashTape.updateEvents = function(artist) {
	var eventbox = document.getElementById("event-box");

	while (eventbox.getRowCount() > 0)
		eventbox.removeItemAt(0);
		var UpdatingMessage = strings.GetStringFromName("UpdatingMessage");

	eventbox.appendItem(UpdatingMessage);
	var req = new XMLHttpRequest();
	req.open("GET", this.lastFMURL("events", "artist", artist), true);
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			/* This is where the goodness happens.  Yay goodness. */
			var xmlobj = (new DOMParser()).parseFromString(this.responseText,
					"text/xml");
			var root = xmlobj.getElementsByTagName("rss")[0];
			var ch = root.getElementsByTagName("channel")[0];
			var items = ch.getElementsByTagName("item");
			eventbox.removeItemAt(0);
			thedeck = document.getElementById('events-wikipedia-deck');
			if (items.length == 0) {
				this.eventsFound = false;
				thedeck.setAttribute("selectedIndex", 1);
			} else {
				thedeck = document.getElementById('events-wikipedia-deck');
				thedeck.setAttribute("selectedIndex", 0);
				this.eventsFound = true;
			}
			for (i=0; (i<items.length); i++) {
				item = items[i];
				var title =
					item.getElementsByTagName("title")[0].firstChild.nodeValue;
				var isodate =
					item.getElementsByTagName("xcal:dtstart")
					[0].firstChild.nodeValue;
				var url = 
					item.getElementsByTagName("link")[0].firstChild.nodeValue;
				if (title.match(/ at (.*) on [0-9]{1,2} /)) {
					venue = RegExp.$1;
				} else {
					title.match(/(.*) on [0-9]{1,2} /);
					venue = RegExp.$1;
				}

				isodate.match(/^[\d]{4}-([\d]{2})-([\d]{2})T/);
				date = RegExp.$1 + "/" + RegExp.$2;

				li = document.createElement('listitem');
				datelc = document.createElement('listcell');
				datelc.setAttribute('label', date);
				li.appendChild(datelc);

				venuelc = document.createElement('listcell');
				venuelc.setAttribute('label', venue);
				li.appendChild(venuelc);

				li.setAttribute('href', url);
				li.appendChild(venuelc);
				li.addEventListener('dblclick', openVenueLink, true);

				eventbox.appendChild(li);
			}
		}
	}
	req.send(null);
}

function openVenueLink() {
	mashTape_openNewLink(this.getAttribute('href'));
}

mashTape.updateFlickrFeed = function(artist) {
	var req = new XMLHttpRequest();
	
	// Check to see if the user specified search or sort type
	var prefs = Cc[PREFSERVICE].getService(Ci.nsIPrefBranch2);
	var flickrSort;
	var flickrSearchType;
	var sortI, searchI;
	try {
		sortI = prefs.getIntPref(flickrSortPref);
		searchI = prefs.getIntPref(flickrSearchTypePref);
	} catch (e) {
		// No preference saved
		sortI = 1;
		searchI = 1;
	}

	switch (sortI) {
		case 2:
			flickrSort = "&sort=date-posted-desc";
			break;
		case 3:
			flickrSort = "&sort=date-taken-desc";
			break;
		case 4:
			flickrSort = "&sort=interestingness-desc";
			break;
		case 1:
		default:
			flickrSort = "&sort=relevance";
	}

	switch (searchI) {
		case 2:
			flickrSearchType = "&tag_mode=all&tags=";
			break;
		case 3:
			flickrSearchType = "&tag_mode=any&tags=";
			break;
		case 1:
		default:
			flickrSearchType = "&text=";
	}

	// Check to see if the user specified to limit photos
	var prefs = Cc[PREFSERVICE].getService(Ci.nsIPrefBranch2);
	var flickrLimit;
	try {
		flickrLimit = prefs.getIntPref(flickrLimitPref);
		if (flickrLimit <= 0)
			flickrLimit = 100;
	} catch (e) {
		// No preference saved
		flickrLimit = 100;
	}

	var url = "http://api.flickr.com/services/rest/?" +
				"method=flickr.photos.search&api_key=" + flickrKey +
				"&per_page=" + flickrLimit + flickrSort + flickrSearchType;
	if (searchI == 1)
		req.open("GET", url + escape(artist) + "%20concert" , true);
	else
		req.open("GET", url + escape(artist) + ",concert" , true);
	// save a reference to our mashTape object to save the Flickr photos to
	req.mT = this;
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			var x = (new DOMParser()).parseFromString(this.responseText,
					"text/xml");
			this.mT.photos = x.getElementsByTagName("photo");
			/*
			if (this.mT.photoWidths)
				delete this.mT.photoWidths;
			this.mT.photoWidths = new Array();
			*/
			this.mT.pIdx = 0;

			var flickrBox = document.getElementById("flickr-box");
			var fiframe = document.getElementById("flickr-browser");
			var fdoc = fiframe.contentWindow.document;
			
			if (verticalOrient) {
				flickrBox.style.minWidth = "150px";
				flickrBox.style.maxWidth = "150px";
				flickrBox.style.width = "150px";
			} else {
				flickrBox.style.minHeight = "150px";
				flickrBox.style.maxHeight = "150px";
				flickrBox.style.height = "150px";
			}

			// Clear out the doc of any existing children first
			while (fdoc.firstChild) {
				fdoc.removeChild(fdoc.firstChild);
			}

			var fdiv = fdoc.createElement("div");
			fdiv.style.position = "relative";
			if (verticalOrient) {
				fdiv.style.height = 24000;
			} else {
				fdiv.style.width = 24000;
			}
			fdoc.appendChild(fdiv);
			
			var ful = fdoc.createElement("ul");
			ful.style.position = "absolute";
			ful.style.top = 0;
			ful.style.left = 0;
			ful.style.padding = 0;
			ful.style.margin = 0;
			ful.style.listStyle = "none";
			fdiv.appendChild(ful);

			this.mT.totalWidth = 0;
			for (i=0; i<this.mT.photos.length; i++) {
				var p = this.mT.photos[i];
				var id = p.getAttribute("id");
				var owner = p.getAttribute("owner");
				var secret = p.getAttribute("secret");
				var server = p.getAttribute("server");
				var farm = p.getAttribute("farm");
				var title = p.getAttribute("title");
				var imgURL = "http://farm" + farm + ".static.flickr.com/" +
					server + "/" + id + "_" + secret + "_m.jpg";
				var aURL = "http://www.flickr.com/photos/" + owner + "/" + id;
			
				var fimg = fdoc.createElement("img");
				fimg.src = imgURL;
				fimg.style.borderStyle = "none";
				fimg.setAttribute("onmouseover",
						"this.height='146';this.style.borderStyle='solid';" +
						"this.style.borderColor='white';" +
						"this.style.borderWidth='1px'");
				fimg.setAttribute("onmouseout",
						"this.height='148';this.style.borderStyle='none';");
				fimg.setAttribute("onclick", "Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow('Songbird:Main').window.gBrowser.loadOneTab('" + aURL + "');");
				if (verticalOrient) {
					// Fix the width to always fit in our iframe
					var owidth = fimg.width;
					var scale = 148 / fimg.width;
					var oheight = fimg.height;
					var nheight = oheight * scale;
					fimg.width = "148";
					// this.mT.photoWidths[i] = parseInt(nwidth);
					this.mT.totalWidth += parseInt(fimg.height);
				} else {
					// Fix the height to always fit in our iframe
					var oheight = fimg.height;
					var scale = 148 / fimg.height;
					var owidth = fimg.width;
					var nwidth = owidth * scale;
					fimg.height = "148";
					// this.mT.photoWidths[i] = parseInt(nwidth);
					this.mT.totalWidth += parseInt(fimg.width);
				}
				var fli = fdoc.createElement("li");
				fli.style.cssFloat = "left";
				fli.style.padding = 0;
				fli.style.margin = 0;

				fli.appendChild(fimg);
				ful.appendChild(fli);
			}
			this.mT.flickrUL = document.getElementById("flickr-browser")
				.contentWindow.document.getElementsByTagName("ul")[0];
			if (verticalOrient) {
				this.mT.flickrWidth = 0 - this.mT.flickrUL.clientHeight;
			} else {
				this.mT.flickrWidth = 0 - this.mT.flickrUL.clientWidth;
			}

			// Check to see if the user wants smooth scroll
			var prefs = Cc[PREFSERVICE].getService(Ci.nsIPrefBranch2);
			try {
				smoothScroll = prefs.getBoolPref(flickrScrollPref);
			} catch (e) {
				// No preference saved
				smoothScroll = false;
			}

			if (smoothScroll) {
				flickrInterval = sflickrInterval;
				flickrShift = sflickrShift;
			} else {
				flickrInterval = cflickrInterval;
				flickrShift = cflickrShift;
				/*
				flickrIdx = 0;
				flickrShift = this.mT.photoWidths[flickrIdx++];
				if (flickrIdx > mashTape.photoWidths.length)
					flickrIdx = 0;
					*/
			}
			
			// Start the slideshow
			this.mT.timerFn =
				setTimeout("startSlideshow(flickrInterval, flickrShift)", 1000);
		} 
	}
	req.send(null);
}

function hoverImage(img) {
	img.style.borderWidth = "2px";
	img.style.borderColor = "white";
	img.style.borderStyle = "solid";
}

function startSlideshow(interval, shift) {
	if (verticalOrient) {
		var l = parseInt(mashTape.flickrUL.style.top);
		if (l < mashTape.flickrWidth) {
			// Scrolled off the right hand side, reset
			flickrIdx = 0;
			mashTape.flickrUL.style.top = 0;
		} else if (l > 0) {
			// Scrolled off the left hand side, stop
			flickrIdx = 0;
			mashTape.flickrUL.style.top = 0;
			clearTimeout(mashTape.timerFn);
			return;
		} else {
			mashTape.flickrUL.style.top = l + shift;
			/*
			if (smoothScroll)
			else {
				mashTape.flickrUL.style.top = l - 150;
							// mashTape.photoWidths[flickrIdx++];
				if (flickrIdx >= mashTape.photoWidths.length)
					flickrIdx = 0;
			}
			*/
		}
	} else {
		var l = parseInt(mashTape.flickrUL.style.left);
		if (l < mashTape.flickrWidth) {
			// Scrolled off the right hand side, reset
			flickrIdx = 0;
			mashTape.flickrUL.style.left = 0;
		} else if (l > 0) {
			// Scrolled off the left hand side, stop
			flickrIdx = 0;
			mashTape.flickrUL.style.left = 0;
			clearTimeout(mashTape.timerFn);
			return;
		} else {
			mashTape.flickrUL.style.left = l + shift;
			/*
			if (smoothScroll)
			else {
				mashTape.flickrUL.style.left = l - 150;
							// mashTape.photoWidths[flickrIdx++];
				if (flickrIdx >= mashTape.photoWidths.length)
					flickrIdx = 0;
			}
			*/
		}
	}
	mashTape.timerFn = setTimeout("startSlideshow(" + interval + ","
			+ shift + ")", interval);
}

function rewFlickr(over) {
	clearTimeout(mashTape.timerFn);
	if (over) {
		startSlideshow(5, 40);
	} else {
		startSlideshow(flickrInterval, flickrShift);
	}
}

function ffwdFlickr(over) {
	clearTimeout(mashTape.timerFn);
	if (over) {
		startSlideshow(5, -40);
	} else {
		startSlideshow(flickrInterval, flickrShift);
	}
}

mashTape.updateTags = function(artist, album, track) {
	var tagbox = document.getElementById("tag-box");
	while (tagbox.getRowCount() > 0)
		tagbox.removeItemAt(0);
		var UpdatingMessage = strings.GetStringFromName("UpdatingMessage");
	tagbox.appendItem(UpdatingMessage);
	var req = new XMLHttpRequest();
	req.open("GET", this.lastFMURL("toptags", "track", artist, track), true);
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			/* This is where the goodness happens.  Yay goodness. */
			var xmlobj = (new DOMParser()).parseFromString(this.responseText,
					"text/xml");
			var root = xmlobj.getElementsByTagName("toptags")[0];
			var tags = root.getElementsByTagName("tag");
			tagbox.removeItemAt(0);
			for (i=0; (i<10 && i<tags.length); i++) {
				tag = tags[i];
				var name =
					tag.getElementsByTagName("name")[0].firstChild.nodeValue;
				var url =
					tag.getElementsByTagName("url")[0].firstChild.nodeValue;
				tagbox.appendItem(name, url);
			}
		} else {
			var mterror = strings.GetStringFromName("mterror");
				var mtretrieving = strings.GetStringFromName("mtretrieving");
					var mttags = strings.GetStringFromName("mttags");
			tagbox.appendItem(mterror);
			tagbox.appendItem(mtretrieving);
			tagbox.appendItem(mttags);
		}
	}
	req.send(null);
}

mashTape.loadLyrics = function() {
	lyricsURL = this.lyricURLs[this.currentLyricsIndex];
	this.currentLyricsIndex++;
	
	var lyrics = document.getElementById("lyrics-box");
	var req = new XMLHttpRequest();
	req.open("GET", lyricsURL, true);
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			/* This is where the goodness happens.  Yay goodness. */
			if (this.responseText.match(/(<div id="SongText">[\s\S]*)/))
				r = RegExp.$1;
			else {
			
				var UnableToFind = strings.GetStringFromName("UnableToFind");
				lyrics.value =UnableToFind;
				return;
			}
			if (r.match(
				/<\/div>([\s\S]*)<\/div>[\s\n]+<div id="LinkAdvert">/))
				r = RegExp.$1;
			else {
				lyrics.value =UnableToFind;
				return;
			}
			// Trim leading whitespace
			r = r.replace(/^[\n\s]+/g,"");
			// Convert HTML <br> tags
			r = r.replace(/<br[\s]*\/>/g, "\n");
			r = r.replace(/<BR[\s]*\/>/g, "\n");
			r = r.replace(/\n\n/g, "\n");
			lyrics.value = r;
		} 
	}
	req.send(null);
}

mashTape.updateLyrics = function(artist, track) {
	var attemptToUpdate = strings.GetStringFromName("attemptToUpdate");
	var lyrics = document.getElementById("lyrics-box");
	lyrics.value = attemptToUpdate;//"Attempting to update lyrics from metrolyrics.com..";
	var req = new XMLHttpRequest();
	var url =
		"http://www.metrolyrics.com/search.php?category=artisttitle&search=";
	req.open("GET", url + escape(artist) + "+" + escape(track), true);
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			/* This is where the goodness happens.  Yay goodness. */
			this.responseText.match(/<tbody class="Clickable">([\s\S]*)/);
			results = RegExp.$1;
			var re = /First"><a href="([^"]+)"/g;
			var linkage;
			var i=0;
			delete mashTape.lyricURLs;
			mashTape.lyricURLs = new Array();
			while (linkage = re.exec(results))
				mashTape.lyricURLs.push(linkage[1]);
			if (mashTape.lyricURLs.length > 0) {
				mashTape.currentLyricsIndex = 0;
				mashTape.loadLyrics();
			} else {
				var UnableToFind = strings.GetStringFromName("UnableToFind");
				lyrics.value = UnableToFind;
				return;
			}
		} 
	}
	req.send(null);
}

function mashTape_nextLyricsMatch() {
	mashTape.loadLyrics();
}

mashTape.updateYouTube = function(artist) {
	var req = new XMLHttpRequest();
	var url = "http://gdata.youtube.com/feeds/videos?vq=%22";
	req.open("GET", url + escape(artist) + "%22", true);
	// save a reference to our mashTape object to save the Flickr photos to
	req.mT = this;
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			var xmlobj = (new DOMParser()).parseFromString(this.responseText,
					"text/xml");
			var feed = xmlobj.getElementsByTagName("feed")[0];
			var entries = feed.getElementsByTagName("entry");
			var utube = document.getElementById("youtube-browser");
			var udoc = utube.contentWindow.document;
			
			// Clear any existing children
			while (udoc.firstChild) {
				udoc.removeChild(udoc.firstChild);
			}

			// Create the html node
			var uhtml = udoc.createElement("html");
			udoc.appendChild(uhtml);

			// Create the head node
			var uhead = udoc.createElement('head');
			uhtml.appendChild(uhead);
			
			// Create our style node
			var ustyles = udoc.createElement("style");
			ustyles.setAttribute("type", "text/css");
			var utStyle;
			if (verticalOrient) {
				var iframeHeight = window.innerHeight - 20;
				utStyle = udoc.createTextNode(
					'body { display:inline; padding: 0px; margin: 0px; }\n' + 
					'.utd { overflow:scroll;max-width:220px;width:220px;min-width:220px;height:400px;max-height:400px;min-height:400px;padding:0px;margin:0px;}\n' +
					'.ut { font-family:helvetica; font-size:.7em; float:left;' +
						'text-align:center; padding:0px; margin-bottom:5px;' +
						'min-width:200px; width:200px; max-width:200px;' +
						'min-height:170px; height:170px; }\n' +
					'.uttitle { padding:0px 0px 4px 0px; margin:0px; ' +
						'color: #dddddd;}');
			} else {
				utStyle = udoc.createTextNode(
					'body { display:inline; padding: 0px; margin: 0px; }\n' + 
					'.utd {overflow:scroll;max-height:177px;height:177px; }\n'+ 
					'.ut { font-family:helvetica; font-size:.7em; float:left;' +
						'text-align:center; padding:0px; margin-bottom:5px;' +
						'min-width:200px; width:200px; max-width:200px;' +
						'min-height:170px; height:170px; }\n' +
					'.uttitle { padding:0px 0px 4px 0px; margin:0px; ' +
						'color: #dddddd;}');
			}
			ustyles.appendChild(utStyle);
			uhead.appendChild(ustyles);

			// Create our Javascript node
			var uscript = udoc.createElement("script");
			uscript.setAttribute("language", "Javascript");
			uscript.setAttribute("type", "text/javascript");
			// Need to break out of the iframe and get back up to the parent
			// window
			var uScriptPause = udoc.createTextNode(
				"function mypause() { " +
				"var gPPS = Components.classes[\"@songbirdnest.com/Songbird/PlaylistPlayback;1\"].getService(Components.interfaces.sbIPlaylistPlayback);\n" +
				"if (gPPS.playing) gPPS.pause(); }");
			uscript.appendChild(uScriptPause);
			uhead.appendChild(uscript);

			// Create our body node
			var ubody = udoc.createElement("body");
			uhtml.appendChild(ubody);

			// Create our div to put all the YouTube Videos in
			var udiv = udoc.createElement("div");
			udiv.className = "utd"
			udiv.style.position = "relative";
			ubody.appendChild(udiv);

			// Check to see if the user specified to limit videos
			var prefs = Cc[PREFSERVICE].getService(Ci.nsIPrefBranch2);
			var youTubeLimit;
			try {
				youTubeLimit = prefs.getIntPref(youTubeLimitPref);
				if (youTubeLimit <= 0)
					youTubeLimit = entries.length;
			} catch (e) {
				// No preference saved
				youTubeLimit = entries.length;
			}
			for (i=0; i<youTubeLimit; i++) {
				var e = entries[i];
				// Only show ones for which we can embed content
				if (e.getElementsByTagName("media:content").length > 0 &&
						e.getElementsByTagName("yt:noembed").length == 0) {
					var title = e.getElementsByTagName("media:title")[0]
									.firstChild.nodeValue;
					var url = e.getElementsByTagName("media:content")[0]
									.getAttribute("url");

					var uembed = udoc.createElement("embed");
					uembed.src = url;
					uembed.setAttribute("type",
							"application/x-shockwave-flash");
					uembed.setAttribute("wmode", "transparent");
					uembed.setAttribute("onclick", "mypause();");
					uembed.width = "198";
					uembed.height = "145";

					var uvidp = udoc.createElement("p");
					uvidp.className = "uttitle";
					uvidp.innerHTML = title;
					
					var uviddiv = udoc.createElement("div");
					uviddiv.className = "ut";
					uviddiv.appendChild(uvidp);
					uviddiv.appendChild(uembed);

					udiv.appendChild(uviddiv);
				}
			}
		} 
	}
	req.send(null);
}


// Use Yahoo's REST search to query for the artist restricted to Wikipedia
mashTape.updateWikipedia = function(artist) {
	var req = new XMLHttpRequest();
	var url = "http://api.search.yahoo.com/WebSearchService/V1/webSearch" +
		"?appid=YahooDemo&query=%22" + escape(artist) +
		"%22&site=wikipedia.org";
	req.open("GET", url, true);
	// save a reference to our mashTape object to save the Flickr photos to
	req.mT = this;
	req.artist = artist;
	var wikiframe = document.getElementById("wikipedia-iframe");
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			var xmlobj = (new DOMParser()).parseFromString(this.responseText,
					"text/xml");
			var feed = xmlobj.getElementsByTagName("ResultSet")[0];
			// Take the first result, it's probably the canonical one we want
			var first = feed.getElementsByTagName("Result")[0];
			if (first == null)
				return;
			url = first.getElementsByTagName("Url")[0].firstChild.nodeValue;
			var pgUrl = url;
			url.match(/http:\/\/en\.wikipedia\.org\/wiki\/(.*)/);
			var pageName = RegExp.$1;
			url = "http://en.wikipedia.org/w/api.php?action=query&prop=" +
					"revisions&titles=" + pageName +
					"&rvprop=content&format=xml";
			
			// Got our Wikipedia URL, now we need to grab that puppy too
			var wkreq = new XMLHttpRequest();
			wkreq.open("GET", url, true);
			wkreq.mT = this.mT;
			wkreq.artist = this.artist;
			wkreq.url = pgUrl;
			wkreq.onreadystatechange = function() {
				if (this.readyState != 4)
					return;
				if (this.status == 200) {
					var xmlobj = (new DOMParser()).parseFromString(
									this.responseText, "text/xml");
					var rev = xmlobj.getElementsByTagName("api")[0].
						getElementsByTagName("query")[0].
						getElementsByTagName("pages")[0].
						getElementsByTagName("page")[0].
						getElementsByTagName("revisions")[0].
						getElementsByTagName("rev")[0].firstChild.nodeValue;
					var validWiki = true;
					if (rev.match(/Years_active\s*=\s*(.*)/))
					{
						var r = RegExp.$1;
						r = r.replace(/\[\[/, "");
						r = r.replace(/\]\]/, "");
						var start;
						var end;
						if (r.match(/([0-9]{4})–(.*)/)) {
							start = RegExp.$1;
							end = RegExp.$2;
						} else if (r.match(/([0-9]{4}).*/)) {
							start = RegExp.$1;
							end = "Present";
						}
					} else {
						validWiki = false;
					}
					var noevents = strings.GetStringFromName("noevents");
					var first = strings.GetStringFromName("firstformed");
					var listed = strings.GetStringFromName("stilllisted");
					var perform = strings.GetStringFromName("performed");
					var to = strings.GetStringFromName("performedto");

					var wikidoc = wikiframe.contentWindow.document;
					var wikibody = wikidoc.body;
					// Remove any existing contents
					while (wikibody.firstChild)
						wikibody.removeChild(wikibody.firstChild);
					wikibody.style.fontFamily = "helvetica, sans";
					wikibody.style.fontSize = "10pt";

					var wikip = wikidoc.createElement("p");
					var wikia = wikidoc.createElement("a");
					wikia.setAttribute("href", "javascript:void(0)");
					wikia.setAttribute("onclick", "Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow('Songbird:Main').window.gBrowser.loadOneTab('" + this.url + "');");
					wikia.style.textDecoration = "underline";
					var artistname = wikidoc.createTextNode(this.artist);
					wikia.appendChild(artistname);
					wikip.appendChild(wikia);

					var txt1;

					start = start.replace(/[\[\]]/g, "");
					end = end.replace(/[\[\]]/g, "");
					if (validWiki) {
						if (end.match(/sent/)) {
							txt1 = wikidoc.createTextNode(" " + noevents +
								" " + first + " " + start + " " + listed);
						} else {
							txt1 = wikidoc.createTextNode(" " + noevents +
								" " + perform + " " + start + " " + to + " "
								+ end);
						}
					} else {
						txt1 = wikidoc.createTextNode(" " + noevents);
					}
					wikip.appendChild(txt1);
					wikibody.appendChild(wikip);
				}
			}
			wkreq.send(null);
		} 
	}
	req.send(null);
}

mashTape.updateArtistBio = function(artist) {
	var postdata = '<?xml version="1.0" encoding="UTF-8"?>' +
		'<methodCall><methodName>artistMetadata</methodName><params>' +
		'<param><value><string>' + artist + '</string></value></param>' +
		'<param><value><string>en</string></value></param></params>' +
		'</methodCall>';
	var req = new XMLHttpRequest();
	req.mT = this;
	req.open("POST", "http://ws.audioscrobbler.com/1.0/rw/xmlrpc.php", true);
	req.setRequestHeader("Content-type", "text/xml;charset=utf-8");
	req.setRequestHeader("Content-length", postdata.length);
	req.setRequestHeader("Connection", "close");
	req.onreadystatechange = function() {
		if (this.readyState != 4)
			return;
		if (this.status == 200) {
			var xmlobj = (new DOMParser()).parseFromString(this.responseText,
					"text/xml");
			var data = xmlobj.getElementsByTagName("methodResponse")[0].
						getElementsByTagName("params")[0].
						getElementsByTagName("param")[0].
						getElementsByTagName("value")[0].
						getElementsByTagName("struct")[0];
			var bioText = data.getElementsByTagName("member")[3].
						getElementsByTagName("value")[0].
						getElementsByTagName("string")[0];
			var bioLink = data.getElementsByTagName("member")[5].
						getElementsByTagName("value")[0].
						getElementsByTagName("string")[0];
			var bioPhoto = data.getElementsByTagName("member")[6].
						getElementsByTagName("value")[0].
						getElementsByTagName("string")[0];

			var box = document.getElementById("artist-bio-box");
			var photo = document.getElementById("artist-photo");
			var button = document.getElementById("artist-bio-button");
			if (bioText.firstChild) {
				box.value = bioText.firstChild.nodeValue.
								replace(/\[\/?[^\]]*\]/g, "");
				photo.src = bioPhoto.firstChild.nodeValue;
				this.mT.bioLink = bioLink.firstChild.nodeValue;
			} else {
				photo.src = "";
				box.value = "No artist bio found.";
				button.disabled = true;
			}
		}
	}
	req.send(postdata);
}

function mashTape_openArtistBio() {
	mashTape_openNewLink(mashTape.bioLink);
}

function mashTape_openNewLink(uri) {
	// Open a new tab
	Cc["@mozilla.org/appshell/window-mediator;1"].
		getService(Ci.nsIWindowMediator).
		getMostRecentWindow("Songbird:Main").window.gBrowser.
		loadOneTab(uri);
}

var mashObserver = {
	mT: mashTape,
	observe: function(subject, topic, data) {
		// topic is the event, i.e. 'metadata.title' or 'metadata.album'
		// data is the actual track name, or album name
		this.mT.updateInfo(topic, data);
	}
}
mashTape._artistDataremote = SB_NewDataRemote("metadata.artist", null);
mashTape._artistDataremote.bindObserver(mashObserver, true);
mashTape._titleDataremote = SB_NewDataRemote("metadata.title", null);
mashTape._titleDataremote.bindObserver(mashObserver, true);
mashTape._albumDataremote = SB_NewDataRemote("metadata.album", null);
mashTape._albumDataremote.bindObserver(mashObserver, true);

function mashTape_onUnload(aEvent) {
	window.removeEventListener("unload", mashTape_onUnload, false);
}

// Initialise the state, get the playlist/playback service
var pps = Components
				.classes['@songbirdnest.com/Songbird/PlaylistPlayback;1']
				.getService(Components.interfaces.sbIPlaylistPlayback);

if (pps.paused || pps.playing) {
	// We're either playing media, or are paused in the middle of playing
	// media - so get the current metadata and force a manual refresh of
	// the info
	var currentGUID = pps.currentGUID;
	var artist = pps.core.getMetadata("artist");
	var track = pps.core.getMetadata("title");
	var album = pps.core.getMetadata("album");

	// Manually trigger the first updates so we're not sitting blank
	setTimeout(function() {
		mashTape.updateInfo("metadata.artist", artist);
		mashTape.updateInfo("metadata.album", album);
		mashTape.updateInfo("metadata.title", track);
	}, 3000);
}

function mashTape_onLoad(aEvent) {
	// Reflow the XUL if necessary.  Our default is to default to the playlist
	// bottom display pane which is horizontal.  If our height is greater than
	// our width, then we need to reflow for a vertical orientation (perhaps
	// we're in the right browser sidebar)
	if (window.innerHeight > window.innerWidth) {
		document.getElementById("tab.events.photos").setAttribute("orient",
				"vertical");
		document.getElementById("tab.events.photos").setAttribute("pack",
				"center");
		document.getElementById("event-box").style.width="180px";
		document.getElementById("wikipedia-iframe").style.width="180px";
		document.getElementById("flickr-box").setAttribute("orient",
				"vertical");
		document.getElementById("flickr-box").style.height="500px";
		document.getElementById("youtube-browser").style.width="240px";
		document.getElementById("tab.artist.bio").setAttribute("orient",
				"vertical");
		document.getElementById("tab.lyrics.tags").setAttribute("orient",
				"vertical");
		document.getElementById("mashtape-tabs").style.display = "none";
		document.getElementById("mashtape-vbuttons").style.display = "block";
		document.getElementById("mashtape-vbuttons").setAttribute("align",
				"center");
		document.getElementById("mashtape-vbuttons").setAttribute("pack",
				"center");
		verticalOrient = true;
	}
}

function mashTape_switchTab(i) {
	document.getElementById("mashtape-tabbox").selectedIndex = i;
}

window.addEventListener("load", mashTape_onLoad, false);
