Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://app/jsmodules/kPlaylistCommands.jsm");

const Cu = Components.utils;
const Cc = Components.classes;
const Ci = Components.interfaces;

var gOS    = null;
var gPrefs = null;

function NowPlayingCommands()
{
  gOS = Cc["@mozilla.org/observer-service;1"]
        .getService(Ci.nsIObserverService);
  gPrefs = Cc["@mozilla.org/preferences-service;1"]
           .getService(Ci.nsIPrefBranch);
  
  if (gOS.addObserver) {
    gOS.addObserver(this, "playlist-commands-ready", false);
    gOS.addObserver(this, "playlist-commands-shutdown", false);
  }
}

NowPlayingCommands.prototype.constructor = NowPlayingCommands;

NowPlayingCommands.prototype = {
  classDescription: "Now Playing Commands",
  classID:          Components.ID("{216eae6f-2d48-46f6-9d6c-a17698bf7085}"),
  contractID:       "@j.c.m/NowPlaying/Commands;1", 

  _commands        : null,
  _mainLibraryGUID : null,
  
  _init: function() {
    
    const PlaylistCommandsBuilder = new Components.
      Constructor("@songbirdnest.com/Songbird/PlaylistCommandsBuilder;1", 
                  "sbIPlaylistCommandsBuilder");
                                  
    this._commands = new PlaylistCommandsBuilder();
    
    // Create the command
    this._commands.
           appendAction(null,
                        "nowplaying_enqueue",
                        "Add to Queue",
                        "Add to Queue",
                        NowPlayingCmds_Enqueue_OnTrigger);
                         
    
    
    this._mainLibraryGUID =
      gPrefs.getComplexValue("songbird.library.main",
                             Ci.nsISupportsString);
    
    var commandsManager = Cc["@songbirdnest.com/Songbird/PlaylistCommandsManager;1"]
                            .getService(Ci.sbIPlaylistCommandsManager);
    
    /* See bug 7837:
       http://bugzilla.songbirdnest.com/show_bug.cgi?id=7873 */
    var registeredCommands = 
      commandsManager.getPlaylistCommandsMediaItem(this._mainLibraryGUID, "simple");
    if (!registeredCommands || !registeredCommands.hasMoreElements()) {
      this._commands.insertPlaylistCommandsBefore(null,
                                                  "nowplaying_enqueue",
                                                  "nowplaying_defaultset",
                                                  commandsManager.request(kPlaylistCommands.MEDIAITEM_DEFAULT));
    }
    
    commandsManager.registerPlaylistCommandsMediaItem(this._mainLibraryGUID, 
                                                      "", 
                                                      this._commands);
  },
  
  _shutdown: function() {

    var commandsManager = Cc["@songbirdnest.com/Songbird/PlaylistCommandsManager;1"]
                          .getService(Ci.sbIPlaylistCommandsManager);

    commandsManager.unregisterPlaylistCommandsMediaItem(this._mainLibraryGUID, 
                                                        "", 
                                                        this._commands);
    this._commands.shutdown();
    this._commands = null;
  },
  
  // nsIObserver
  observe: function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case "playlist-commands-ready":
        gOS.removeObserver(this, "playlist-commands-ready");
        if (aData == "default")
          this._init();
        break;
      case "playlist-commands-shutdown":
        gOS.removeObserver(this, "playlist-commands-shutdown");
        if (aData == "default")
          this._shutdown();
        
        gOS    = null;
        gPrefs = null;
        break;
    }
  },

  QueryInterface:
    XPCOMUtils.generateQI([Ci.nsIObserver]) 
};

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([NowPlayingCommands],
  function(aCompMgr, aFileSpec, aLocation) {
    XPCOMUtils.categoryManager.addCategoryEntry(
      "app-startup",
      NowPlayingCommands.prototype.classDescription,
      "service," + NowPlayingCommands.prototype.contractID,
      true,
      true);
  });
}

// Called when Add to queue is triggered
function NowPlayingCmds_Enqueue_OnTrigger(aContext, aSubMenuId, aCommandId, aHost) {
  var view = unwrap(aContext.playlist).getListView();
  var sel = view.selection.selectedIndexedMediaItems;
  var unwrapper = {
    _enumerator : null,
    hasMoreElements : function() {
      return this._enumerator.hasMoreElements();
    },
    getNext : function() {
      var item = this._enumerator.getNext().mediaItem;
      return item;
    },
    QueryInterface : function(iid) {
      if (iid.equals(Components.interfaces.nsISimpleEnumerator) ||
          iid.equals(Components.interfaces.nsISupports))
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
  }
  var npl = Cc["@j.c.m/NowPlaying/List;1"].getService().wrappedJSObject;
  unwrapper._enumerator = sel;
  npl.enqueue(unwrapper);
}

function unwrap(obj) {
  if (obj && obj.wrappedJSObject)
    obj = obj.wrappedJSObject;
  return obj;
}
