Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://app/jsmodules/sbLibraryUtils.jsm");
Components.utils.import("resource://app/jsmodules/sbProperties.jsm");
Components.utils.import("resource://app/jsmodules/kPlaylistCommands.jsm");
Components.utils.import("resource://app/jsmodules/ArrayConverter.jsm");

const NowPlayingModes = {
  BASIC  : 0,
  CUSTOM : 1,
  PARTY  : 2
};

const NOWPLAYING_LIST_CONTRACTID = "@j.c.m/NowPlaying/List;1";
const NOWPLAYING_LIST_CLASSNAME = "Now Playing List";
const NOWPLAYING_LIST_CID = Components.ID("{7ce97f4b-a1bd-4a90-b53c-49d5b4cd0e1a}");

const SONGBIRD_DATAREMOTE_CONTRACTID = "@songbirdnest.com/Songbird/DataRemote;1";

const SONGBIRD_STRINGBUNDLE = "chrome://songbird/locale/songbird.properties";
const JCM_STRINGBUNDLE = "chrome://jcm-nowplaying/locale/nowplaying.properties";

const LIST_GUID_PREF     = "extensions.nowplaying@j.c.m.listguid";
const HIDE_NODE_PREF     = "extensions.nowplaying@j.c.m.hidenode";
const SHOW_CONTENT_PREF  = "extensions.nowplaying@j.c.m.showcontent";
const FORK_PREF          = "extensions.nowplaying@j.c.m.fork";
const START_MODE_PREF    = "extensions.nowplaying@j.c.m.startmode";
const PLAY_ON_QUEUE_PREF = "extensions.nowplaying@j.c.m.playonqueue";

const Cu = Components.utils;
const Cc = Components.classes;
const Ci = Components.interfaces;


var gOS    = null;
var gPrefs = null;
var gPPS   = null;

function NowPlayingList()
{
  gOS    = Cc["@mozilla.org/observer-service;1"]
           .getService(Ci.nsIObserverService);
  gPrefs = Cc["@mozilla.org/preferences-service;1"]
           .getService(Ci.nsIPrefBranch);
  
  if (gOS.addObserver) {
    gOS.addObserver(this, "profile-after-change", false);
    gOS.addObserver(this, "profile-before-change", false);
    gOS.addObserver(this, "xul-window-visible", false);
  }
  
  this.wrappedJSObject = this;
}


// COMPONENT

NowPlayingList.prototype.constructor = NowPlayingList;

NowPlayingList.prototype = {
  classDescription : NOWPLAYING_LIST_CLASSNAME,
  classID          : NOWPLAYING_LIST_CID,
  contractID       : NOWPLAYING_LIST_CONTRACTID,
  
  _xpcom_factory  :
  {
    singleton: null,
    createInstance: function (aOuter, aIID)
    {
      if (aOuter != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      if (this.singleton == null)
        this.singleton = new NowPlayingList();
      return this.singleton.QueryInterface(aIID);
    }
  },
  
  _xpcom_categories : [{
   // Each object in the array specifies the parameters to pass to
   // nsICategoryManager.addCategoryEntry(). 'true' is passed for
   // both aPersist and aReplace params.
   category: "app-startup",
   
   // optional, defaults to the object's classDescription
   //entry: "entry name",
   
   // optional, defaults to the object's contractID (unless
   // 'service' is specified)
   //value: "...",
   
   // optional, defaults to false. When set to true, and only if 'value'
   // is not specified, the concatenation of the string "service," and the
   // object's contractID is passed as aValue parameter of addCategoryEntry.
   service: true
 }],
  
  
  
  _list         : null,
  _mode         : null,
  _playlist     : null,
  
  // basic mode attaches a listener to track filter and sort changes
  _isListener   : false,
  
  _timer        : null,
  
  // Data Remotes
  _label        : null,
  _length       : null,
  _canSave      : null,
  _canRandomise : null,
  _canClear     : null,
  
  // fix for timing conflicts between bindPlaylist and _init
  _initialised     : false,
  _playlistWaiting : null,
    
  get list() {
    return this._list;
  },
  
  get mode() {
    return this._mode;
  },
  
  get playlist() {
    return this._playlist;
  },
  
  
  _init : function() {
    if (this._initialised) return;
    /* Setup list */
    var guid = gPrefs.getCharPref(LIST_GUID_PREF);
    var library = LibraryUtils.mainLibrary;
    try {
      var item = library.getMediaItem(guid);
      var list = item.QueryInterface(Ci.sbIMediaList);
      this._list = list;
    } catch (e) {
      // Doesn't exist
      var list = library.createMediaList("simple");
      list.name = getString("playlistname", "Now playing");
      // save the media list
      gPrefs.setCharPref(LIST_GUID_PREF, list.guid);
      // set up default column specifications
      var columnSpec = SBProperties.trackName + " 170 " +
                       SBProperties.rating + " 80";
      list.setProperty(SBProperties.columnSpec, columnSpec);
      list.setProperty(SBProperties.defaultColumnSpec, columnSpec);
      this._list = list;
    }
    
    this._timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    
    // attach Data Remotes
    var createDataRemote = new Components.Constructor( SONGBIRD_DATAREMOTE_CONTRACTID, Ci.sbIDataRemote, "init");
    this._label        = createDataRemote("jcm.nowplaying.label", null);
    this._length       = createDataRemote("jcm.nowplaying.length", null);
    this._canSave      = createDataRemote("jcm.nowplaying.cansave", null);
    this._canRandomise = createDataRemote("jcm.nowplaying.canrandomise", null);
    this._canClear     = createDataRemote("jcm.nowplaying.canclear", null);
    
    // attach listeners
    gPPS.addListener(this);
    
    // initialise
    this._mode = gPrefs.getIntPref(START_MODE_PREF);
    
    this._initialised = true;
    if (this._playlistWaiting) this.bindPlaylist(this._playlistWaiting);
  },
  
  _init2 : function() {
    try {
      // set up the service pane node for the list
      var LSPS  = Components.classes['@songbirdnest.com/servicepane/library;1']
                  .getService(Components.interfaces.sbILibraryServicePaneService);
      
      var node = LSPS.getNodeForLibraryResource(this._list);
      if (node) {
        node.hidden = gPrefs.getBoolPref("extensions.nowplaying@j.c.m.hidenode");
      } else {
        throw("Error: Couldn't find a service pane node for the list we just created\n");
      }
    } catch (e) {
      dump("ERROR: Now Playing List init2() - " + e);
    }
  },
  
  _deinit : function() {
    this._list.clear();
    this._list = null;
    
    this._timer = null;
    
    // release Data Remotes
    this._label.unbind();
    this._length.unbind();
    this._canSave.unbind();
    this._canRandomise.unbind();
    this._canClear.unbind();
    
    // release listeners
    gPPS.removeListener(this);
    
    // if haven't already
    if (this._playlist) this.unbindPlaylist();
    
    // save prefs
    gPrefs.setIntPref(START_MODE_PREF, this._mode);
  },
  
  
  
  bindPlaylist : function(aPlaylist) {
    if (!this._initialised) {
      this._playlistWaiting = aPlaylist;
      return;
    }
    
    if (this._playlist) return;
    
    this._playlist = aPlaylist;
    // attach listener
    this._playlist.addEventListener("playlist-play", this._onPlaylistPlay, false);
    switch (this._mode) {
    case NowPlayingModes.BASIC :
      this._toBasic();
      break;
    case NowPlayingModes.CUSTOM :
      this._toCustom();
      break;
    case NowPlayingModes.PARTY :
      // TODO
      break;
    }
    
    this._update();
    
    this._playlistWaiting = null;
  },
  
  unbindPlaylist : function() {
    // release listeners
    this._playlist.removeEventListener("playlist-play", this._onPlaylistPlay, false);
    if (this._isListener) {
      this._isListener = false;
      gPPS.playingView.removeListener(this);
    }
    
    this._playlist = null;
  },
  
  
  
  setMode : function(newMode) {
    if (this._isListener) {
      this._isListener = false;
      gPPS.playingView.removeListener(this);
    }
    switch (newMode) {
    case NowPlayingModes.BASIC :
      this._toBasic();
      break;
    case NowPlayingModes.CUSTOM :
      this._toCustom();
      break;
    case NowPlayingModes.PARTY :
      // TODO
    default :
      this.setMode(NowPlayingModes.BASIC);
      break;
    }
    this._update();
  },
  
  /*
  Basic mode binds whatever media list view playing to the now playing list.
  If there is nothing playing or the show content pref is off, it will empty
  the custom media list and bind that to the playlist.
  */
  _toBasic : function() {
    this._mode = NowPlayingModes.BASIC;
    
    if (gPPS.playing && gPPS.playingView.mediaList.equals(this._currentList()))
      return;
    
    if (gPPS.playing && gPrefs.getBoolPref(SHOW_CONTENT_PREF)) {
      // can't use cloneView() because it keeps filter constraints
      var view = gPPS.playingView.mediaList.createView();
      this._bindView(view);
      
      gPPS.playingView.addListener(this)
      this._isListener = true;
      this._syncSearchAndFilterConstraints(gPPS.playingView);
    } else {
      this._list.clear();
      
      var list = this._currentList();
      if (list && list.equals(this._list)) return;
      
      var view = this._list.createView();
      this._bindView(view);
    }
  },
  
  /*
  Custom mode binds a 'hidden' playlist to the now playing list which can be
  edited 'on the go'.
  */
  _toCustom : function() {
    this._mode = NowPlayingModes.CUSTOM;
    
    var list = this._currentList();
    if (list && list.equals(this._list)) return;
    
    var view = this._list.createView();
    this._bindView(view);
  },
  
  _bindView : function(aView) {
    if (!this._playlist) return;
    
    var commandsMgr = Cc["@songbirdnest.com/Songbird/PlaylistCommandsManager;1"]
                        .createInstance(Ci.sbIPlaylistCommandsManager);
    var commands = commandsMgr.request(kPlaylistCommands.MEDIAITEM_DEFAULT);
    this._playlist.bind(aView, commands);
    this._playlist.getListView().selection.selectNone()
  },
  
  _fillList : function() {
    this._list.clear();
    
    if (!gPPS.playing) return;
    if (gPPS.playingView.mediaList.equals(this._list)) return;
    
    var view = gPPS.playingView;
    var items = [];
    for (var i = 0; i < view.length; i++) {
      // javascript is slow ...
      items.push(view.getItemByIndex(i));
    }
    var enum = ArrayConverter.enumerator(items);
    this._list.addSome(enum);
  },
  
  
  
  /* sbIPlaylistPlaybackListener */
  onStop: function() {
    // only do this if playback has really stopped
    this._timer.init(this, 500, Ci.nsITimer.TYPE_ONE_SHOT);
  },
  _stopTimeout: function() {
    if (gPPS.playing) return;
    
    this._list.clear();
    
    var list = this._currentList();
    if (list && list.equals(this._list)) return;
      
    var view = this._list.createView();
    this._bindView(view);
    this._update();
  },
  
  /* sbIPlaylistPlaybackListener */
  onBeforeTrackChange: function(aItem, aView, aIndex) { },
  
  /* sbIPlaylistPlaybackListener */
  onTrackChange: function(aItem, aView, aIndex) {
    if (this._mode == NowPlayingModes.BASIC) {
      // will return if there's nothing to do
      // otherwise binds the correct media list view to the playlist\
      // if it changed
      this._toBasic();
    }
    this._update();
  },
  
  
  
  /* sbIMediaListViewListener */
  onFilterChanged: function(aChangedView) {
    this._update();
    this._syncSearchAndFilterConstraints(aChangedView);
  },
  
  /* sbIMediaListViewListener */
  onSearchChanged: function(aChangedView) {
    this._update();
    this._syncSearchAndFilterConstraints(aChangedView);
  },
  
  /* sbIMediaListViewListener */
  onSortChanged: function(aChangedView) { },
  
  
  
  _update : function() {
    switch (this._mode) {
    case NowPlayingModes.BASIC :
      this._canSave.boolValue      = false;
      this._canRandomise.boolValue = false;
      this._canClear.boolValue     = false;
      
      if (gPPS.playing) {
        var view = gPPS.playingView;
        this._label.stringValue = view.mediaList.name;
        this._length.stringValue = formatLength(view.length);
        
        if (gPrefs.getBoolPref(SHOW_CONTENT_PREF)) {
          this._canRandomise.boolValue = true;
          this._canClear.boolValue     = true;
        }
      } else {
        this._label.stringValue = "";
        this._length.stringValue = "";
      }
      break;
    case NowPlayingModes.CUSTOM :
      this._label.stringValue = getString("customplaylist", "Custom playlist");
      this._length.stringValue = formatLength(this._list.length);
      
      this._canSave.boolValue      = true;
      this._canRandomise.boolValue = true;
      this._canClear.boolValue     = true;
      break;
    }
  },
  
  _syncSearchAndFilterConstraints : function(aSyncView) {
    var list  = this._currentList();
    if (!list) return;
    if (list.equals(this._list)) return;
    
    try {
      var search = aSyncView.searchConstraint;
      var filter = aSyncView.filterConstraint;
      
      var view = this._playlist.getListView();
      view.searchConstraint = search;
      view.filterConstraint = filter;
    } catch (e) {
      dump("Now Playing: syncSearchAndFilterConstraints - " + e);
    }
  },
  
  
  
  /* Actions */
  
  save : function() {
    // Don't use LibraryUtils as we don't know which library the playlist is
    // from
    var library = this._currentList().library;
    var newList = library.copyMediaList("simple", this._currentList());
    newList.name = getString("customplaylist", "Custom playlist");
    
    var LSPS  = Cc['@songbirdnest.com/servicepane/library;1']
                .getService(Ci.sbILibraryServicePaneService);
    var node = LSPS.getNodeForLibraryResource(newList);
    node.name = getString("customplaylist", "Custom playlist");
    
    // We don't ask for a new name anymore
  },
  
  randomise : function() {
    if ((this._mode == NowPlayingModes.BASIC) && 
        gPrefs.getBoolPref(FORK_PREF)) {
      this._fillList();
      this._toCustom();
      this._update();
    }
    
    var mediaList = this._currentList();
    var n = mediaList.length;
    var orderable = mediaList.QueryInterface(Ci.sbIOrderableMediaList);
    while (--n >= 0) {
      var k = Math.round(n*Math.random());
      orderable.moveLast(k);
    }
    
    // wierd stuff gets selected after randomising
    this._playlist.getListView().selection.selectNone();
  },
  
  clear : function() {
    if (this._mode == NowPlayingModes.CUSTOM) {
      this._list.clear();
    } else {
      if (gPrefs.getBoolPref(FORK_PREF)) {
        // the clear list button should be disabled if show contents is not on
        this._list.clear();
        this._toCustom();
      } else {
        this._currentList().clear();
      }
    }
    this._update();
  },
  
  enqueue : function(items) {
    switch (this._mode) {
    case NowPlayingModes.BASIC :
      this._toCustom();
      // fall through
    case NowPlayingModes.CUSTOM :
      this._list.addSome(items);
      break;
    }
    this._update();
    
    if (gPrefs.getBoolPref(PLAY_ON_QUEUE_PREF)) {
      if (!gPPS.playing || !gPPS.playingView.mediaList.equals(this._list)) {
        if (this._playlist) {
          gPPS.playView(this._playlist.getListView(), 0);
        } else {
          var view = this._list.createView();
          gPPS.playView(view, 0);
        }
      }
    }
  },
  
  
  
  _currentList : function() {
    try {
      return this._playlist.getListView().mediaList;
    } catch (e) {
      return null;
    }
  },
  
  
  
  /* Event Handlers */
  
  _onPlaylistPlay : function(aEvent) {
    var target = aEvent.target;
    if (target.wrappedJSObject) {
      target = target.wrappedJSObject;
    }
    var index = target.tree.currentIndex;
    if (index < 0) {
      index = 0;
    }
    if (index < target.mediaListView.length) {
      gPPS.playView(target.mediaListView, index);
    }
  },
  
  
  
  // watch for XRE startup and shutdown messages 
  observe: function(subject, topic, data) {
    switch (topic) {
    case "timer-callback" :
      this._stopTimeout();
      this._timer.cancel();
      break;
    case "profile-after-change":
      gOS.removeObserver(this, "profile-after-change");
      
      /* Other services */
      gPPS = Cc["@songbirdnest.com/Songbird/PlaylistPlayback;1"]
             .getService(Ci.sbIPlaylistPlayback);
      // Preferences are initialized, ready to start the service
      this._init();
      break;
    case "xul-window-visible" :
      gOS.removeObserver(this, "xul-window-visible");
      this._init2();
      break;
    case "profile-before-change":
      gOS.removeObserver(this, "profile-before-change");
      this._deinit();
      
      // Release Services to avoid memory leaks
      gOS    = null;
      gPrefs = null;
      gPPS   = null;
      break;
    }
  },
  
  QueryInterface:
    XPCOMUtils.generateQI([Ci.nsIObserver, Ci.sbIPlaylistPlaybackListener, Ci.sbIMediaListViewListener]) 
};



// XPCOM Registration

/*function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([NowPlayingList],
  function(aCompMgr, aFileSpec, aLocation) {
    XPCOMUtils.categoryManager.addCategoryEntry(
      "app-startup",
      NowPlayingList.prototype.classDescription,
      "service," + NowPlayingList.prototype.contractID,
      true,
      true);
  });
}*/
function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([NowPlayingList]);
}


// helper functions

function getString(aName, aDefaultValue) {
  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"]
                            .getService(Ci.nsIStringBundleService);
  
  var sbBundle = stringBundleService.createBundle(SONGBIRD_STRINGBUNDLE);
  try {
    return sbBundle.GetStringFromName(aName);
  } catch (e) {} // do nothing, try extension string bundle
  
  var jcmBundle = stringBundleService.createBundle(JCM_STRINGBUNDLE);
  try {
    return jcmBundle.GetStringFromName(aName);
  } catch(e) {
    return aDefaultValue;
  }
}

function formatLength(aLength) {
  switch (aLength) {
    case 0 :
      return getString("library.noitem", "no items"); break;
    case 1 :
      return "1 " + getString("library.oneitem", "item"); break;
    case "" :
      return ""; break;
    default :
      return aLength + " " + getString("library.manyitems", "items"); break;
  }
}
