try {
  var browser;
  var artistdataremote;
  var timeout;
  var on_artist_change;
  var lasturl;

  function initWikipedia() {
    
    lasturl = "";
    artistdataremote = SB_NewDataRemote("metadata.artist", null);
    on_artist_change = { 
      observe: function( aSubject, aTopic, aData ) {
        onArtistChange();
      } 
    };
    artistdataremote.bindObserver(on_artist_change, true);
    
    browser = document.getElementById("wikipedia.browser");
    
    timeout = setTimeout(loadArtist, 1000);
  }

  function resetWikipedia() {
    browser = null;
    if (timeout) {
      clearTimeout(timeout);
      timeout = null;
    }
    artistdataremote.unbind();
    artistdataremote = null;
    on_artist_change = null;
  }

  function loadArtist() {
    var url;
    var artist = artistdataremote.stringValue;
    var keyword = "";
    timeout = null;
    var words = artist.split(" ");
    var dobreak = false;
    for (var i=0;!dobreak && i<words.length;i++) {
      var word = words[i];
      var subwords = word.split("_");
      for (var j=0;!dobreak && j<subwords.length;j++) {
        var subword = subwords[j];
        if (subword.charAt(subword.length-1) == '.') subword = subword.slice(0, subword.length-1); // remove final '.'
        if (subword.charAt(subword.length-1) == '-') subword = subword.slice(0, subword.length-1); // remove final '-'
        var e = excludedChars(subword);
        if (e > -1) { 
          subword = subword.substr(0, e);
          dobreak = true; 
          if (e == 0) break;
        }
        if (keyword != "") keyword += "_";
        keyword += subword.slice(0, 1).toUpperCase() + subword.slice(1).toLowerCase();
      }
    }
    if (keyword == "") keyword = "Songbird_(software)";
    url = "http://en.wikipedia.org/wiki/" + escape(keyword);
    if (url == lasturl) return;
    lasturl = url;
    browser.loadURI(url, null, null);
  }

  function onArtistChange() {
    if (timeout) clearTimeout(timeout);
    timeout = setTimeout(loadArtist, 1000);
  }

  function excludedChars(str) {
    for (var i=0;i<str.length;i++) {
      var c = str.charCodeAt(i);
      if (c == 0x21) continue; // allow '!'
      if (c == 0x2D) continue; // allow '-'
      if (c == 0x27) continue; // allow "'"
      if (c >= 0x30 && c <= 0x93) continue; // allow 0 to 9
      // disallow non characters
      if (c < 0x41) return i;
      if (c > 0x5A && c < 0x61) return i;
      if (c > 0x7A && c < 0xC0) return i;
    }
    return -1;
  }
} catch (e) {
}
